# Copyright (c) 2014 Fatih Erikli
# The MIT License (MIT)
# Permission is hereby granted, free of charge, to any person obtaining a copy of
# this software and associated documentation files (the "Software"), to deal in
# the Software without restriction, including without limitation the rights to
# use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
# the Software, and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""  A class that parses nginx configuration with pyparsing """


import string
import copy
from insights.contrib.pyparsing import (
    Literal, White, Word, alphanums, CharsNotIn, Combine, Forward, Group,
    Optional, OneOrMore, ZeroOrMore, Regex, stringEnd, restOfLine)


def create_parser():
    # constants
    space = Optional(White())
    nonspace = Regex(r"\S+")
    left_bracket = Literal("{").suppress()
    right_bracket = space.leaveWhitespace() + Literal("}").suppress()
    semicolon = Literal(";").suppress()
    key = Word(alphanums + "_/+-.")
    dollar_var = Combine(Literal('$') + Regex(r"[^\{\};,\s]+"))
    condition = Regex(r"\(.+\)")
    # Matches anything that is not a special character, and ${SHELL_VARS}, AND
    # any chars in single or double quotes
    # All of these COULD be upgraded to something like
    # https://stackoverflow.com/a/16130746
    dquoted = Regex(r'(\".*\")')
    squoted = Regex(r"(\'.*\')")
    nonspecial = Regex(r"[^\{\};,]")
    varsub = Regex(r"(\$\{\w+\})")
    # nonspecial nibbles one character at a time, but the other objects take
    # precedence.  We use ZeroOrMore to allow entries like "break ;" to be
    # parsed as assignments
    value = Combine(ZeroOrMore(dquoted | squoted | varsub | nonspecial))

    location = CharsNotIn("{};," + string.whitespace)
    # modifier for location uri [ = | ~ | ~* | ^~ ]
    modifier = Literal("=") | Literal("~*") | Literal("~") | Literal("^~")

    # rules
    comment = space + Literal('#') + restOfLine

    assignment = space + key + Optional(space + value, default=None) + semicolon
    location_statement = space + Optional(modifier) + Optional(space + location + space)
    if_statement = space + Literal("if") + space + condition + space
    charset_map_statement = space + Literal("charset_map") + space + value + space + value

    map_statement = space + Literal("map") + space + nonspace + space + dollar_var + space
    # This is NOT an accurate way to parse nginx map entries; it's almost
    # certainly too permissive and may be wrong in other ways, but it should
    # preserve things correctly in mmmmost or all cases.
    #
    #    - I can neither prove nor disprove that it is correct wrt all escaped
    #      semicolon situations
    # Addresses https://github.com/fatiherikli/nginxparser/issues/19
    map_pattern = Regex(r'".*"') | Regex(r"'.*'") | nonspace
    map_entry = space + map_pattern + space + value + space + semicolon
    map_block = Group(
        Group(map_statement).leaveWhitespace() +
        left_bracket +
        Group(ZeroOrMore(Group(comment | map_entry)) + space).leaveWhitespace() +
        right_bracket)

    block = Forward()

    # key could for instance be "server" or "http", or "location" (in which case
    # location_statement needs to have a non-empty location)

    block_begin = (Group(space + key + location_statement) ^
                   Group(if_statement) ^
                   Group(charset_map_statement)).leaveWhitespace()

    block_innards = Group(ZeroOrMore(Group(comment | assignment) | block | map_block) + space).leaveWhitespace()

    block << Group(block_begin + left_bracket + block_innards + right_bracket)

    script = OneOrMore(Group(comment | assignment) ^ block ^ map_block) + space + stringEnd
    return script.parseWithTabs().leaveWhitespace()


class UnspacedList(list):
    """After handling by create_parser(), there is white space existing in the list. Use this class to wrap a list
     [of lists], making any whitespace entries magically invisible"""

    def __init__(self, list_source):
        # ensure our argument is not a generator, and duplicate any sublists
        self.spaced = copy.deepcopy(list(list_source))
        self.dirty = False

        # Turn self into a version of the source list that has spaces removed
        # and all sub-lists also UnspacedList()ed
        list.__init__(self, list_source)
        for i, entry in reversed(list(enumerate(self))):
            if isinstance(entry, list):
                sublist = UnspacedList(entry)
                list.__setitem__(self, i, sublist)
                self.spaced[i] = sublist.spaced
            elif self._spacey(entry):
                list.__delitem__(self, i)

    def _spacey(self, x):
        return (isinstance(x, str) and x.isspace()) or x == ''



"""
DumpE2FS - Command ``dumpe2fs -h``
==================================

This parser handles ``dumpe2fs`` output.

The object provides access to this data using a dictionary.  Particular keys
are stored as lists:

* Filesystem features
* Filesystem flags
* Default mount options

Other keys are stored as strings.  The name of the device is stored in the
``dev_name`` property.

Typical contents of the ``/sbin/dumpe2fs -h /dev/device`` command::

    dumpe2fs 1.41.12 (17-May-2010)
    Filesystem volume name:   <none>
    Last mounted on:          /usr
    Filesystem UUID:          1b332c5d-2410-4934-9118-466f8a14841f
    Filesystem magic number:  0xEF53
    Filesystem revision #:    1 (dynamic)
    Filesystem features:      has_journal ext_attr resize_inode dir_index filetype needs_recovery extent flex_bg sparse_super large_file huge_file uninit_bg dir_nlink extra_isize
    Filesystem flags:         signed_directory_hash
    Default mount options:    user_xattr acl

Examples:

    >>> e2fs = shared[DumpE2fs]
    >>> e2fs.dev_name
    '/dev/device'
    >>> e2fs['Filesystem volume name']
    '<none>'
    >>> e2fs['Last mounted on']
    '/usr'
    >>> e2fs['Filesystem UUID']
    '1b332c5d-2410-4934-9118-466f8a14841f'
    >>> e2fs['Filesystem magic number']
    '0xEF53'
    >>> e2fs['Filesystem revision #']
    '1 (dynamic)'
    >>> e2fs['Filesystem features']
    ['has_journal', 'ext_attr', 'resize_inode', 'dir_index', 'filetype',
     'needs_recovery', 'extent', 'flex_bg', 'sparse_super', 'large_file',
     'huge_file', 'uninit_bg', 'dir_nlink', 'extra_isize'],
    >>> e2fs['Filesystem flags']
    ['signed_directory_hash']
    >>> e2fs['Default mount options']
    ['user_xattr', 'acl']


"""

from .. import parser, LegacyItemAccess, CommandParser
from insights.specs import Specs

COMPOUND_FIELDS = ['Filesystem features', 'Filesystem flags', 'Default mount options']


@parser(Specs.dumpe2fs_h)
class DumpE2fs(CommandParser, LegacyItemAccess):
    """
    Parse each line in the output of the ``dumpe2fs`` command.
    """
    def parse_content(self, content):
        dumpe2fs_values_dict = {}
        for line in content:
            if ":" in line:
                key, value = line.split(":", 1)
                if key in COMPOUND_FIELDS:
                    dumpe2fs_values_dict[key] = list(value.strip().split())
                else:
                    dumpe2fs_values_dict[key] = value.strip()
        self.dev_name = self.file_name.split('dumpe2fs_-h_')[-1].replace('.', '/')
        self.data = dumpe2fs_values_dict



import json
import pytest

try:
    from unittest.mock import patch
    builtin_open = "builtins.open"
except Exception:
    from mock import patch
    builtin_open = "__builtin__.open"

from os import path

from insights.core.exceptions import SkipComponent, ContentException
from insights.specs.datasources.leapp import leapp_report


with open(path.join(path.dirname(__file__), 'leapp-report.json'), 'r') as fp:
    INPUT_OK = json.load(fp)

with open(path.join(path.dirname(__file__), 'leapp-report.ret'), 'r') as fp:
    RESULT = fp.readlines()

INPUT_NG_1 = json.loads("""
{ "entries":[
  {
    "a": "B"
  }
 ]
}
""".strip())

INPUT_NG_2 = json.loads("{}")


@patch("json.load", return_value=INPUT_OK)
@patch("os.path.isfile", return_value=True)
@patch(builtin_open)
def test_leapp_report_ok(m_open, m_isfile, m_load):
    result = leapp_report({})
    result_json = json.loads(''.join(result.content).strip())
    expected_json = json.loads(''.join(RESULT).strip())
    for ret in result_json:
        assert ret in expected_json


@patch("json.load", return_value=INPUT_NG_1)
@patch("os.path.isfile", return_value=True)
@patch(builtin_open)
def test_leapp_report_nothing(m_open, m_isfile, m_load):
    with pytest.raises(SkipComponent) as ce:
        leapp_report({})
    assert "Nothing" in str(ce)


@patch("json.load", return_value=INPUT_NG_2)
@patch("os.path.isfile", return_value=True)
@patch(builtin_open)
def test_leapp_report_ng_2(m_open, m_isfile, m_load):
    with pytest.raises(ContentException) as ce:
        leapp_report({})
    assert "Nothing" in str(ce)


@patch("os.path.isfile", return_value=False)
def test_leapp_report_no_file(isfile):
    with pytest.raises(SkipComponent):
        leapp_report({})



from insights.parsers import checkin_conf
from insights.parsers.checkin_conf import CheckinConf
from insights.tests import context_wrap
from insights.tests.parsers import ic_testmod


CONFIG = """
[logging]
config = /etc/splice/logging/basic.cfg

# this is used only for single-spacewalk deployments
[spacewalk]
# Spacewalk/Satellite server to use for syncing data.
host=
# Path to SSH private key used to connect to spacewalk host.
ssh_key_path=
login=swreport

# these are used for multi-spacewalk deployments
# [spacewalk_one]
# type = ssh
# # Spacewalk/Satellite server to use for syncing data.
# host=
# # Path to SSH private key used to connect to spacewalk host.
# ssh_key_path=
# login=swreport
#
# [spacewalk_two]
# type = file
# # Path to directory containing report output
# path = /path/to/output

[katello]
hostname=localhost
port=443
proto=https
api_url=/sam
admin_user=admin
admin_pass=admin
#autoentitle_systems = False
#flatten_orgs = False
""".strip()


def test_checkin_conf():
    result = CheckinConf(context_wrap(CONFIG))

    assert list(result.sections()) == ['logging', 'spacewalk', 'katello']
    assert result.get('spacewalk', 'host') == ''


def test_checkin_conf_doc_examples():
    env = {
        'CheckinConf': CheckinConf,
        'checkin_conf': CheckinConf(context_wrap(CONFIG))
    }
    failed, total = ic_testmod(checkin_conf, globs=env)
    assert failed == 0



import doctest

from insights.parsers import leapp
from insights.parsers.leapp import LeappReport
from insights.tests import context_wrap
from insights.tests.datasources.test_leapp import RESULT


def test_leapp():
    ret = LeappReport(context_wrap(RESULT, path='insights_commands/leapp_report'))
    assert len(ret) == 4
    assert ret[0]['title'] == "Use of NFS detected. Upgrade can't proceed"
    assert 'summary' in ret[0]
    assert ret[1]['title'] == "Possible problems with remote login using root account"
    assert 'summary' in ret[1]
    assert ret[0]['remediations'][0]['context'] == "Disable NFS temporarily for the upgrade if possible."
    assert 'If you depend on remote' in ret[1]['remediations'][0]['context']
    assert 'remediations' not in ret[2]
    assert 'external' in ret[2]
    assert 'remediations' not in ret[3]
    assert 'related_resources' in ret[3]


def test_doc_examples():
    env = {'leapp_report': LeappReport(context_wrap(RESULT))}
    failed, total = doctest.testmod(leapp, globs=env)
    assert failed == 0



"""
Subscription manager release commands
=====================================

Shared parser for parsing output of the ``subscription-manager release``
commands.

SubscriptionManagerReleaseShow - command ``subscription-manager release --show``
--------------------------------------------------------------------------------

"""
from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.subscription_manager_release_show)
class SubscriptionManagerReleaseShow(CommandParser):
    """
    Class for parsing the output of `subscription-manager release --show` command.

    Typical output of the command is::

        Release: 7.2

    Attributes:
        set (str): the set release.
        major (int): the major version of the set release.
        minor (int): the minor version of the set release.

    Examples:
        >>> type(rhsm_rel)
        <class 'insights.parsers.subscription_manager_release.SubscriptionManagerReleaseShow'>
        >>> rhsm_rel.set
        '7.2'
        >>> rhsm_rel.major
        7
        >>> rhsm_rel.minor
        2
    """

    def parse_content(self, content):
        self.set = self.major = self.minor = None
        l = len(content)
        if l != 1:
            raise SkipComponent("Content takes at most 1 line ({0} given).".format(l))
        line = content[0].strip()
        if line != 'Release not set':
            line_splits = line.split()
            # Release: 6.7 or Release 7Server
            if len(line_splits) == 2:
                rel = line_splits[-1]
                rel_splits = rel.split('.')
                # Release: 6.7
                if len(rel_splits) == 2:
                    if rel_splits[0].isdigit() and rel_splits[-1].isdigit():
                        self.set = rel
                        self.major = int(rel_splits[0])
                        self.minor = int(rel_splits[-1])
                        return
                # Release: 7Server
                elif rel.endswith('Server') and rel[0].isdigit():
                    self.set = rel
                    self.major = int(rel[0])
                    # leave self.minor as None
                    return
                # Release: 8
                elif rel[0].isdigit() and len(rel) == 1:
                    self.set = rel
                    self.major = int(rel[0])
                    # leave self.minor as None
                    return

            raise SkipComponent("Incorrect content: {0}".format(line))
        # Release not set



import platform
import sys
from collections import OrderedDict

import yaml as pyyaml


_items = "viewitems" if sys.version_info < (3,) else "items"
_std_dict_is_order_preserving = sys.version_info >= (3, 7) or (
    sys.version_info >= (3, 6) and platform.python_implementation() == "CPython"
)


def map_representer(dumper, data):
    return dumper.represent_dict(getattr(data, _items)())


def map_constructor(loader, node):
    loader.flatten_mapping(node)
    pairs = loader.construct_pairs(node)
    try:
        return OrderedDict(pairs)
    except TypeError:
        loader.construct_mapping(node)  # trigger any contextual error
        raise


_loaders = [getattr(pyyaml.loader, x) for x in pyyaml.loader.__all__]
_dumpers = [getattr(pyyaml.dumper, x) for x in pyyaml.dumper.__all__]
try:
    _cyaml = pyyaml.cyaml.__all__
except AttributeError:
    pass
else:
    _loaders += [getattr(pyyaml.cyaml, x) for x in _cyaml if x.endswith("Loader")]
    _dumpers += [getattr(pyyaml.cyaml, x) for x in _cyaml if x.endswith("Dumper")]

Dumper = None
for Dumper in _dumpers:
    pyyaml.add_representer(dict, map_representer, Dumper=Dumper)
    pyyaml.add_representer(OrderedDict, map_representer, Dumper=Dumper)

Loader = None
if not _std_dict_is_order_preserving:
    for Loader in _loaders:
        pyyaml.add_constructor("tag:yaml.org,2002:map", map_constructor, Loader=Loader)


# Merge PyYAML namespace into ours.
# This allows users a drop-in replacement:
#   import oyaml as yaml
del map_constructor, map_representer, Loader, Dumper
from yaml import *



import doctest
import pytest

from insights.core.exceptions import ParseException
from insights.parsers import sendq_recvq_socket_buffer
from insights.parsers.sendq_recvq_socket_buffer import RecvQSocketBuffer, SendQSocketBuffer
from insights.tests import context_wrap

SENDQ_SOCKET_BUFFER = """
4096	16384	4194304
""".strip()

EMPTY_SENDQ_SOCKET_BUFFER = """
""".strip()

RECVQ_SOCKET_BUFFER = """
4096	87380	6291456
""".strip()

EMPTY_RECVQ_SOCKET_BUFFER = """
""".strip()


def test_empty_sendq_socket_buffer():
    with pytest.raises(ParseException) as exc:
        SendQSocketBuffer(context_wrap(EMPTY_SENDQ_SOCKET_BUFFER))
    assert str(exc.value) == "Empty content"


def test_sendq_socket_buffer():
    sendq_buffer = SendQSocketBuffer(context_wrap(SENDQ_SOCKET_BUFFER))
    assert sendq_buffer.minimum == 4096
    assert sendq_buffer.default == 16384
    assert sendq_buffer.maximum == 4194304
    assert sendq_buffer.raw == '4096 16384 4194304'


def test_empty_recvq_socket_buffer():
    with pytest.raises(ParseException) as exc:
        RecvQSocketBuffer(context_wrap(EMPTY_RECVQ_SOCKET_BUFFER))
    assert str(exc.value) == "Empty content"


def test_recvq_socket_buffer():
    recvq_buffer = RecvQSocketBuffer(context_wrap(RECVQ_SOCKET_BUFFER))
    assert recvq_buffer.minimum == 4096
    assert recvq_buffer.default == 87380
    assert recvq_buffer.maximum == 6291456
    assert recvq_buffer.raw == '4096 87380 6291456'


def test_doc():
    env = {
            'sendq_buffer_values': SendQSocketBuffer(context_wrap(SENDQ_SOCKET_BUFFER)),
            'recvq_buffer_values': RecvQSocketBuffer(context_wrap(RECVQ_SOCKET_BUFFER)),
          }
    failures, tests = doctest.testmod(sendq_recvq_socket_buffer, globs=env)
    assert failures == 0



"""
ceph_insights - command ``ceph insights``
=========================================
"""
import json
import re
from .. import CommandParser, parser
from insights.specs import Specs


@parser(Specs.ceph_insights)
class CephInsights(CommandParser):
    """
    Parse the output of the ``ceph insights`` command.

    Attributes:
        version (dict): version information of the Ceph cluster.
        data (dict): a dictionary of the parsed output.

    The ``data`` attribute is a dictionary containing the parsed output of the
    ``ceph insights`` command. The following are available in ``data``:

    * ``crashes`` - summary of daemon crashes for the past 24 hours
    * ``health`` - the current and historical (past 24 hours) health checks
    * ``config`` - cluster and daemon configuration settings
    * ``osd_dump`` - osd and pool information
    * ``df`` - storage usage statistics
    * ``osd_tree`` - osd topology
    * ``fs_map`` - file system map
    * ``crush_map`` - the CRUSH map
    * ``mon_map`` - monitor map
    * ``service_map`` - service map
    * ``manager_map`` - manager map
    * ``mon_status`` - monitor status
    * ``pg_summary`` - placement group summary
    * ``osd_metadata`` - per-OSD metadata
    * ``version`` - ceph software version
    * ``errors`` - any errors encountered collecting this data

    The ``version`` attribute contains a normalized view of ``self.data["version"]``.

    Examples:
        >>> ceph_insights.version["release"] == 14
        True
        >>> ceph_insights.version["major"] == 0
        True
        >>> ceph_insights.version["minor"] == 0
        True
        >>> isinstance(ceph_insights.data["crashes"], dict)
        True
        >>> isinstance(ceph_insights.data["health"], dict)
        True
    """
    IGNORE_RE = [
         "\*\*\* DEVELOPER MODE",
         "\d+-\d+-\d+.+WARNING: all dangerous"
    ]

    bad_lines = [
        "module 'insights' is not enabled",
        "no valid command found"
    ]

    def __init__(self, *args, **kwargs):
        kwargs.update(dict(extra_bad_lines=self.bad_lines))
        super(CephInsights, self).__init__(*args, **kwargs)

    def _sanitize_content(self, content):
        """Remove lines matching IGNORE_RE at start of content"""
        slice_point = 0
        ignore_re = re.compile('|'.join(CephInsights.IGNORE_RE))
        for line in content:
            if not line or ignore_re.match(line):
                slice_point += 1
                continue
            break
        return content[slice_point:]

    def _parse_version(self):
        """
        Add a Ceph version property as a dictionary with the keys "release",
        "major", "minor" containing numeric values, and the key "full" with the
        full version string. If Ceph is not compiled with verison information
        (this should never be the case in a production system), then "release",
        "major", and "minor" are set to None.
        """
        self.version = {
            "release": None,
            "major": None,
            "minor": None
        }
        self.version.update(self.data["version"])

    def parse_content(self, content):
        content = self._sanitize_content(content)
        self.data = json.loads(''.join(content))
        self._parse_version()



import doctest
from insights.parsers import semanage
from insights.tests import context_wrap

USERS_COUNT_OUTPUT_1 = """
{
    "staff_u": 2
}
""".strip()

USERS_COUNT_OUTPUT_2 = """
{
    "staff_u": 2,
    "test_u": 4
}
""".strip()


def test_docs():
    users = semanage.LinuxUserCountMapSelinuxUser(context_wrap(USERS_COUNT_OUTPUT_1))
    env = {'users': users}
    failed, total = doctest.testmod(semanage, globs=env)
    assert failed == 0


def test_users_count_map_staffu_selinux_user():
    users = semanage.LinuxUserCountMapSelinuxUser(context_wrap(USERS_COUNT_OUTPUT_2))
    assert 'staff_u' in users
    assert users['staff_u'] == 2
    assert 'test_u' in users
    assert users['test_u'] == 4



import pytest
import doctest

from insights.parsers.hostname import Hostname as HnF
from insights import SkipComponent
from insights.parsers.lssap import Lssap
from insights.parsers.saphostctrl import SAPHostCtrlInstances
from insights.combiners import sap
from insights.combiners.sap import Sap
from insights.combiners.hostname import Hostname
from insights.tests import context_wrap
from insights.tests.parsers.test_saphostctrl import SAPHOSTCTRL_HOSTINSTANCES_DOCS, SAPHOSTCTRL_HOSTINSTANCES_GOOD

Lssap_nw_TEST = """
 - lssap version 1.0 -
==========================================
  SID   Nr   Instance    SAPLOCALHOST                        Version                 DIR_EXECUTABLE
  HA2|  16|       D16|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D16/exe
  HA2|  22|       D22|         lu0416|749, patch 10, changelist 1698137|          /usr/sap/HA2/D22/exe
  HA2|  50|       D50|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D50/exe
  HA2|  51|       D51|         lu0418|749, patch 10, changelist 1698137|          /usr/sap/HA2/D51/exe
  HA2|  52|       D52|         lu0418|749, patch 10, changelist 1698137|          /usr/sap/HA2/D52/exe
==========================================
""".strip()

Lssap_hana_TEST = """
 - lssap version 1.0 -
==========================================
  SID   Nr   Instance    SAPLOCALHOST                        Version                 DIR_EXECUTABLE
  HA2|  16|       HDB16|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/HDB16/exe
==========================================
""".strip()

Lssap_ascs_TEST = """
 - lssap version 1.0 -
==========================================
  SID   Nr   Instance    SAPLOCALHOST                        Version                 DIR_EXECUTABLE
  HA2|  16|       ASCS16|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/ASCS16/exe
==========================================
""".strip()

Lssap_all_TEST = """
 - lssap version 1.0 -
==========================================
  SID   Nr   Instance    SAPLOCALHOST                        Version                 DIR_EXECUTABLE
  HA2|  16|       D16|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D16/exe
  HA2|  22|       D22|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D22/exe
  HA2|  16|       HDB16|       lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/HDB16/exe
  HA2|  16|       ASCS16|      lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/ASCS16/exe
==========================================
""".strip()

Lssap_doc_TEST = """
 - lssap version 1.0 -
==========================================
  SID   Nr   Instance    SAPLOCALHOST                        Version                 DIR_EXECUTABLE
  HA2|  16|       D16|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D16/exe
  HA2|  22|       D22|         lu0418|749, patch 10, changelist 1698137|          /usr/sap/HA2/D22/exe
  HA2|  16|       HDB16|       lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/HDB16/exe
==========================================
""".strip()

HOSTNAME = 'lu0417.example.com'
HOSTNAME1 = 'li-ld-1810.example.com'

SAPHOSTCTRL_HOSTINSTANCES_R_CASE = '''
*********************************************************
 SID , String , R4D
 SystemNumber , String , 12
 InstanceName , String , DVEBMGS12
 InstanceType , String , Primary Application Server
 Hostname , String , r4d00
 FullQualifiedHostname , String , r4d00.example.corp
 SapVersionInfo , String , 753, patch 501, changelist 1967207
*********************************************************
 SID , String , R4D
 SystemNumber , String , 10
 InstanceName , String , ASCS10
 InstanceType , String , ABAP Central Services
 Hostname , String , r4d01
 FullQualifiedHostname , String , r4d01.example.corp
 SapVersionInfo , String , 753, patch 501, changelist 1967207
*********************************************************
 SID , String , WDX
 SystemNumber , String , 20
 InstanceName , String , W20
 InstanceType , String , WebDispatcher
 Hostname , String , r4d02
 FullQualifiedHostname , String , host_97.example.corp
 SapVersionInfo , String , 773, patch 121, changelist 1917131
*********************************************************
 SID , String , SMD
 SystemNumber , String , 98
 InstanceName , String , SMDA98
 InstanceType , String , Solution Manager Diagnostic Agent
 Hostname , String , r4d01
 FullQualifiedHostname , String , host_97.example.corp
 SapVersionInfo , String , 745, patch 400, changelist 1734487
*********************************************************
 SID , String , SMD
 SystemNumber , String , 97
 InstanceName , String , SMDA97
 InstanceType , String , Solution Manager Diagnostic Agent
 Hostname , String , r4d00
 FullQualifiedHostname , String , host_97.example.corp
 SapVersionInfo , String , 745, patch 400, changelist 1734487
'''

HOSTNAME2 = 'host_97.example.corp'


def test_lssap_netweaver():
    lssap = Lssap(context_wrap(Lssap_nw_TEST))
    hn = Hostname(HnF(context_wrap(HOSTNAME)), None, None, None)
    sap = Sap(hn, None, lssap)
    assert sap['D50'].number == '50'
    assert 'D16' in sap.local_instances
    assert 'D51' in sap.all_instances
    assert 'D51' not in sap.local_instances
    assert sap.is_netweaver is True
    assert sap.is_hana is False
    assert sap.is_ascs is False


def test_saphostcrtl_hana():
    lssap = Lssap(context_wrap(Lssap_nw_TEST))
    inst = SAPHostCtrlInstances(context_wrap(SAPHOSTCTRL_HOSTINSTANCES_DOCS))
    hn = Hostname(HnF(context_wrap(HOSTNAME)), None, None, None)
    sap = Sap(hn, inst, lssap)
    assert 'D50' not in sap
    assert sap.local_instances == ['HDB88']
    assert 'HDB90' in sap.all_instances
    assert sap['HDB88'].number == '88'
    assert sap['HDB90'].hostname == 'lu0418'
    assert sap['HDB90'].version == '749, patch 211, changelist 1754007'
    assert sap.number('HDB90') == '90'
    assert sap.sid('HDB88') == 'D89'
    assert sap.is_netweaver is False
    assert sap.is_hana is True
    assert sap.is_ascs is False


def test_saphostcrtl_hana_2():
    lssap = Lssap(context_wrap(Lssap_all_TEST))
    inst = SAPHostCtrlInstances(context_wrap(SAPHOSTCTRL_HOSTINSTANCES_GOOD))
    hn = Hostname(HnF(context_wrap(HOSTNAME1)), None, None, None)
    sap = Sap(hn, inst, lssap)
    assert 'D50' not in sap
    assert 'HDB00' in sap
    assert sorted(sap.local_instances) == sorted(['HDB88', 'HDB90', 'SMDA91', 'HDB62', 'HDB00'])
    assert sorted(sap.all_instances) == sorted([
        'ASCS07', 'ASCS52', 'D54', 'DVEBMGS09', 'ERS08', 'HDB00', 'HDB62',
        'HDB88', 'HDB90', 'SCS10', 'SMDA91'])
    assert sorted(sap.business_instances) == sorted([
        'ASCS07', 'ASCS52', 'D54', 'DVEBMGS09', 'ERS08', 'HDB00', 'HDB62',
        'HDB88', 'HDB90', 'SCS10'])
    assert sorted(sap.function_instances) == sorted(['SMDA91'])
    assert sap['HDB88'].number == '88'
    assert sap['HDB90'].hostname == 'li-ld-1810'
    assert sap['DVEBMGS09'].version == '749, patch 301, changelist 1779613'
    assert sap.version('HDB90') == '749, patch 211, changelist 1754007'
    assert sap.hostname('HDB62') == 'd62dbsrv'
    assert sap.type('SCS10') == 'SCS'
    assert sap.full_type('SCS10') == 'Java Central Services'
    assert sap.is_netweaver is True
    assert sap.is_hana is True
    assert sap.is_ascs is True


def test_lssap_hana():
    lssap = Lssap(context_wrap(Lssap_hana_TEST))
    hn = Hostname(HnF(context_wrap(HOSTNAME)), None, None, None)
    sap = Sap(hn, None, lssap)
    assert 'D50' not in sap
    assert sap.is_netweaver is False
    assert sap.is_hana is True
    assert sap.is_ascs is False


def test_lssap_ascs():
    lssap = Lssap(context_wrap(Lssap_ascs_TEST))
    hn = Hostname(HnF(context_wrap(HOSTNAME)), None, None, None)
    sap = Sap(hn, None, lssap)
    assert sap['ASCS16'].sid == 'HA2'
    # ASCS is also a kind of NetWeaver
    assert sap.is_netweaver is True
    assert sap.is_hana is False
    assert sap.is_ascs is True


def test_all():
    lssap = Lssap(context_wrap(Lssap_all_TEST))
    hn = Hostname(HnF(context_wrap(HOSTNAME)), None, None, None)
    sap = Sap(hn, None, lssap)
    assert sap['D16'].version == '749, patch 10, changelist 1698137'
    assert sap['ASCS16'].hostname == 'lu0417'
    assert sap.is_netweaver is True
    assert sap.is_hana is True
    assert sap.is_ascs is True


def test_r_case():
    saphostctrl = SAPHostCtrlInstances(context_wrap(SAPHOSTCTRL_HOSTINSTANCES_R_CASE))
    hn = Hostname(HnF(context_wrap(HOSTNAME2)), None, None, None)
    sap = Sap(hn, saphostctrl, None)
    assert sorted(sap.local_instances) == sorted(['W20', 'SMDA98', 'SMDA97'])
    assert sap['DVEBMGS12'].version == '753, patch 501, changelist 1967207'
    assert sap['ASCS10'].hostname == 'r4d01'
    assert sap.is_netweaver is True
    assert sap.is_hana is False
    assert sap.is_ascs is True


def test_doc_examples():
    env = {
            'saps': Sap(
                Hostname(HnF(context_wrap(HOSTNAME)), None, None, None),
                None,
                Lssap(context_wrap(Lssap_doc_TEST))
            )
          }
    failed, total = doctest.testmod(sap, globs=env)
    assert failed == 0


def test_ab():
    hn = Hostname(HnF(context_wrap(HOSTNAME)), None, None, None)
    with pytest.raises(SkipComponent) as se:
        Sap(hn, None, None)
    assert 'No SAP instance.' in str(se)



"""
Client Metadata Files
=====================

Parsers for files that are generated by the insights-client.

AnsibleHost - File ``ansible_host``
-----------------------------------

BlacklistedSpecs - File ``blacklisted_specs``
---------------------------------------------

BranchInfo - File ``branch_info``
---------------------------------

DisplayName - File ``display_name``
-----------------------------------

MachineID - File ``machine_id``
-------------------------------

Tags - file ``tags.json``
-------------------------

VersionInfo - file ``version_info``
-----------------------------------
"""
from insights.core import Parser, JSONParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.parsers.hostname import HostnameBase
from insights.specs import Specs


@parser(Specs.ansible_host)
class AnsibleHost(HostnameBase):
    """
    Class for parsing the content of ``ansible_host``.

    Typical content::

        host1

    Attributes:
        hostname (str): Ansible Hostname.
        raw (str): RAW content of this file.

    Examples:
        >>> type(ansible_host)
        <class 'insights.parsers.client_metadata.AnsibleHost'>
        >>> ansible_host.hostname == "host1"
        True
        >>> ansible_host.raw == "host1"
        True
    """
    pass


@parser(Specs.blacklisted_specs)
class BlacklistedSpecs(JSONParser):
    """
    Class for parsing the content of ``blacklisted_specs`` or
    ``blacklisted_specs.txt``

    Typical content::

        {"specs": ["dmesg", "fstab"]}

    Attributes:
        specs (list): List of blacklisted specs.

    Examples:
        >>> type(specs)
        <class 'insights.parsers.client_metadata.BlacklistedSpecs'>
        >>> result = ['dmesg', 'fstab']
        >>> specs.specs == result
        True
    """
    @property
    def specs(self):
        return self.data['specs']


@parser(Specs.branch_info)
class BranchInfo(JSONParser):
    """
    Class for parsing the content of ``branch_info`` as a dictionary.

    Typical content::

        {"remote_branch": -1, "remote_leaf": -1}

    Examples:
        >>> type(branch_info)
        <class 'insights.parsers.client_metadata.BranchInfo'>
        >>> branch_info['remote_branch'] == -1
        True
    """
    pass


@parser(Specs.display_name)
class DisplayName(HostnameBase):
    """
    Class for parsing the content of ``display_name``.

    Typical content::

        host1

    Attributes:
        hostname (str): Display Hostname.
        raw (str): RAW content of this file.

    Examples:
        >>> type(display_name)
        <class 'insights.parsers.client_metadata.DisplayName'>
        >>> display_name.hostname == "host1"
        True
        >>> display_name.raw == "host1"
        True
    """
    pass


@parser(Specs.machine_id)
class MachineID(Parser):
    """
    Class for parsing the content of ``display_name``.

    Typical content::

        176843d1-90fa-499b-9f94-111111111111

    Attributes:
        id (str): Machine ID

    Examples:
        >>> type(machine_id)
        <class 'insights.parsers.client_metadata.MachineID'>
        >>> machine_id.id== "176843d1-90fa-499b-9f94-111111111111"
        True

    Raises:
        SkipComponent: Nothing collected
    """
    def parse_content(self, content):
        if not content or len(content) > 1:
            raise SkipComponent()
        self.id = content[0].strip()


@parser(Specs.tags)
class Tags(JSONParser):
    """
    Class for parsing the content of ``tags.json``.

    Typical content::

        [{"key": "group", "value": "_group-name-value_", "namespace": "insights-client"}]

    Attributes:
        data (list): List of parsed dictionaries.

    Examples:
        >>> type(tags)
        <class 'insights.parsers.client_metadata.Tags'>
        >>> tags.data[0]['key'] == "group"
        True
        >>> tags.data[0]['value'] == "_group-name-value_"
        True
        >>> tags.data[0]['namespace'] == "insights-client"
        True
    """
    pass


@parser(Specs.version_info)
class VersionInfo(JSONParser):
    """
    Class for parsing the content of ``version_info``.

    Typical content of this file is::

        {"core_version": "3.0.203-1", "client_version": "3.1.1"}

    .. note::

        The :attr:`client_version` provided by this Parser is a short version
        only, to get the full version of the ``insights-client`` package,
        please use the :class:`insights.parsers.installed_rpms.InstalledRpms`
        Parser instead.

    Examples:
        >>> type(ver)
        <class 'insights.parsers.client_metadata.VersionInfo'>
        >>> ver.core_version == '3.0.203-1'
        True
        >>> ver.client_version == '3.1.1'
        True
    """
    @property
    def core_version(self):
        """
        Returns:
            (str): The version of the insights core.
        """
        return self['core_version']

    @property
    def client_version(self):
        """
        Returns:
            (str): The version of the insights client.

        .. note::

            This attribute returns a short version of the insights client only,
            to get the full version of the ``insights-client`` package, please
            use the :class:`insights.parsers.installed_rpms.InstalledRpms` Parser
            instead.
        """
        return self['client_version']



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import doveconf
from insights.parsers.doveconf import Doveconf
from insights.tests import context_wrap

CONF = """
# 2.2.36 (1f10bfa63): /etc/dovecot/dovecot.conf
# OS: Linux 4.18.0-193.el8.x86_64 x86_64 Red Hat Enterprise Linux release 8.2 (Ootpa) 
# Hostname: localhost
# NOTE: Send doveconf -n output instead when asking for help.
auth_anonymous_username = anonymous
auth_cache_negative_ttl = 1 hours
auth_cache_size = 0
auth_policy_request_attributes = login=%{requested_username} pwhash=%{hashed_password} remote=%{rip} device_id=%{client_id} protocol=%s
auth_policy_server_api_header = 
log_timestamp = "%b %d %H:%M:%S "
login_access_sockets = 
login_greeting = Dovecot ready.
login_log_format = %$: %s
login_log_format_elements = user=<%u> method=%m rip=%r lip=%l mpid=%e %c session=<%{session}>
login_plugin_dir = /usr/lib64/dovecot/login
mail_log_prefix = "%s(%u): "
mdbox_rotate_size = 2 M
mmap_disable = no
namespace inbox {
  disabled = no
  hidden = no
  ignore_on_failure = no
  inbox = yes
  list = yes
  location = 
  mailbox Drafts {
    auto = no
    autoexpunge = 0
    autoexpunge_max_mails = 0
    comment = 
    driver = 
    special_use = \Drafts
  }
  mailbox Junk {
    auto = no
    autoexpunge = 0
    autoexpunge_max_mails = 0
    comment = 
    driver = 
    special_use = \Junk
  }
  mailbox Sent {
    auto = no
    autoexpunge = 0
    autoexpunge_max_mails = 0
    comment = 
    driver = 
    special_use = \Sent
  }
  mailbox "Sent Messages" {
    auto = no
    autoexpunge = 0
    autoexpunge_max_mails = 0
    comment = 
    driver = 
    special_use = \Sent
  }
  mailbox Trash {
    auto = no
    autoexpunge = 0
    autoexpunge_max_mails = 0
    comment = 
    driver = 
    special_use = \Trash
  }
  order = 0
  prefix = 
  separator = 
  subscriptions = yes
  type = private
}
passdb {
  args = 
  auth_verbose = default
  default_fields = 
  deny = no
  driver = pam
  master = no
  mechanisms = 
  name = 
  override_fields = 
  pass = no
  result_failure = continue
  result_internalfail = continue
  result_success = return-ok
  skip = never
  username_filter = 
}
service aggregator {
  chroot = .
  client_limit = 0
  drop_priv_before_exec = no
  executable = aggregator
  extra_groups = 
  fifo_listener replication-notify-fifo {
    group = 
    mode = 0600
    user = 
  }
  group = 
  idle_kill = 0
  privileged_group = 
  process_limit = 0
  process_min_avail = 0
  protocol = 
  service_count = 0
  type = 
  unix_listener replication-notify {
    group = 
    mode = 0600
    user = 
  }
  user = $default_internal_user
  vsz_limit = 18446744073709551615 B
}
service anvil {
  chroot = empty
  client_limit = 0
  drop_priv_before_exec = no
  executable = anvil
  extra_groups = 
  group = 
  idle_kill = 4294967295 secs
  privileged_group = 
  process_limit = 1
  process_min_avail = 1
  protocol = 
  service_count = 0
  type = anvil
  unix_listener anvil-auth-penalty {
    group = 
    mode = 0600
    user = 
  }
  unix_listener anvil {
    group = 
    mode = 0600
    user = 
  }
  user = $default_internal_user
  vsz_limit = 18446744073709551615 B
}
stats_user_min_time = 1 hours
submission_host = 
syslog_facility = mail
userdb {
  args = 
  auth_verbose = default
  default_fields = 
  driver = passwd
  name = 
  override_fields = 
  result_failure = continue
  result_internalfail = continue
  result_success = return-ok
  skip = never
}
valid_chroot_dirs = 
verbose_proctitle = no
verbose_ssl = no
version_ignore = no
""".strip()  # noqa: W291

EMPTY = ""
INVALID = "b{{{la bla foo ha [] ^&*@#$%"


def test_doveconf():
    c = Doveconf(context_wrap(CONF))
    assert c['auth_anonymous_username'].value == 'anonymous'
    assert c['auth_anonymous_username'].line == 'auth_anonymous_username = anonymous'
    assert c['auth_cache_negative_ttl'].value == '1 hours'
    assert c['auth_cache_size'].value == '0'
    assert c['auth_policy_request_attributes'].value == 'login=%{requested_username} pwhash=%{hashed_password} remote=%{rip} device_id=%{client_id} protocol=%s'
    assert c['auth_policy_server_api_header'].value is None
    assert c['log_timestamp'].value == '"%b %d %H:%M:%S "'
    assert c['namespace'][0].value == 'inbox'
    assert c['namespace'][0]["mailbox"][3].value == 'Sent Messages'
    assert c['namespace'][0]["mailbox"][3]["special_use"][0].value == '\\Sent'


def test_empty():
    with pytest.raises(SkipComponent):
        Doveconf(context_wrap(EMPTY))


def test_invalid():
    with pytest.raises(Exception):
        Doveconf(context_wrap(INVALID))


def test_doc_examples():
    env = {
        'doveconf': Doveconf(context_wrap(CONF)),
    }
    failed, total = doctest.testmod(doveconf, globs=env)
    assert failed == 0



from insights.combiners.selinux import SELinux
from insights.parsers.grub_conf import Grub1Config, Grub2Config
from insights.combiners.grub_conf import GrubConf
from insights.parsers.selinux_config import SelinuxConfig
from insights.parsers.sestatus import SEStatus
from insights.tests import context_wrap

GRUB_DISABLED = 'grub_disabled'
GRUB_NOT_ENFORCING = 'grub_not_enforcing'
RUNTIME_DISABLED = 'sestatus_disabled'
RUNTIME_NOT_ENFORCING = 'sestatus_not_enforcing'
BOOT_DISABLED = 'selinux_conf_disabled'
BOOT_NOT_ENFORCING = 'selinux_conf_not_enforcing'

SESTATUS_OUT = """
SELinux status:                 enabled
SELinuxfs mount:                /sys/fs/selinux
SELinux root directory:         /etc/selinux
Loaded policy name:             targeted
Current mode:                   enforcing
Mode from config file:          enforcing
Policy MLS status:              enabled
Policy deny_unknown status:     allowed
Max kernel policy version:      30
"""

SESTATUS_OUT_DISABLED = """
SELinux status:                 disabled
SELinuxfs mount:                /sys/fs/selinux
SELinux root directory:         /etc/selinux
Loaded policy name:             targeted
Current mode:                   enforcing
Mode from config file:          enforcing
Policy MLS status:              enabled
Policy deny_unknown status:     allowed
Max kernel policy version:      30
"""

SESTATUS_OUT_NOT_ENFORCING = """
SELinux status:                 enabled
SELinuxfs mount:                /sys/fs/selinux
SELinux root directory:         /etc/selinux
Loaded policy name:             targeted
Current mode:                   permissive
Mode from config file:          enforcing
Policy MLS status:              enabled
Policy deny_unknown status:     allowed
Max kernel policy version:      30
"""

SELINUX_CONF = """
# This file controls the state of SELinux on the system.
# SELINUX= can take one of these three values:
#     enforcing - SELinux security policy is enforced.
#     permissive - SELinux prints warnings instead of enforcing.
#     disabled - No SELinux policy is loaded.
SELINUX=enforcing
# SELINUXTYPE= can take one of these two values:
#     targeted - Targeted processes are protected,
#     mls - Multi Level Security protection.
SELINUXTYPE=targeted
"""

SELINUX_CONF_DISABLED = """
# This file controls the state of SELinux on the system.
# SELINUX= can take one of these three values:
#     enforcing - SELinux security policy is enforced.
#     permissive - SELinux prints warnings instead of enforcing.
#     disabled - No SELinux policy is loaded.
SELINUX=disabled
# SELINUXTYPE= can take one of these two values:
#     targeted - Targeted processes are protected,
#     mls - Multi Level Security protection.
SELINUXTYPE=targeted
"""

SELINUX_CONF_NOT_ENFORCING = """
# This file controls the state of SELinux on the system.
# SELINUX= can take one of these three values:
#     enforcing - SELinux security policy is enforced.
#     permissive - SELinux prints warnings instead of enforcing.
#     disabled - No SELinux policy is loaded.
SELINUX=permissive
# SELINUXTYPE= can take one of these two values:
#     targeted - Targeted processes are protected,
#     mls - Multi Level Security protection.
SELINUXTYPE=targeted
"""

SELINUX_CONF_COMMENTED = """
# This file controls the state of SELinux on the system.
# SELINUX= can take one of these three values:
#     enforcing - SELinux security policy is enforced.
#     permissive - SELinux prints warnings instead of enforcing.
#     disabled - No SELinux policy is loaded.
# SELINUX=enforcing
# SELINUXTYPE= can take one of these two values:
#     targeted - Targeted processes are protected,
#     mls - Multi Level Security protection.
SELINUXTYPE=targeted
"""

SESTATUS_TEMPLATE = {
    'loaded_policy_name': 'targeted', 'selinux_root_directory': '/etc/selinux',
    'selinuxfs_mount': '/sys/fs/selinux', 'mode_from_config_file': 'enforcing',
    'policy_mls_status': 'enabled',
    'policy_deny_unknown_status': 'allowed', 'max_kernel_policy_version': '30'
}


# rhel-6
GRUB1_TEMPLATE = """
# grub.conf generated by anaconda
#
# Note that you do not have to rerun grub after making changes to this file
# NOTICE:  You have a /boot partition.  This means that
#          all kernel and initrd paths are relative to /boot/, eg.
#          root (hd0,0)
#          kernel /vmlinuz-version ro root=/dev/mapper/VolGroup-lv_root
#          initrd /initrd-[generic-]version.img
#boot=/dev/sda
default=0
timeout=5
splashimage=(hd0,0)/grub/splash.xpm.gz
hiddenmenu
title Red Hat Enterprise Linux 6 (2.6.32-642.el6.x86_64)
	root (hd0,0)
	kernel /vmlinuz-2.6.32-642.el6.x86_64 {kernel_boot_options} ro root=/dev/mapper/VolGroup-lv_root rd_NO_LUKS LANG=en_US.UTF-8 rd_NO_MD rd_LVM_LV=VolGroup/lv_swap SYSFONT=latarcyrheb-sun16 crashkernel=auto rd_LVM_LV=VolGroup/lv_root  KEYBOARDTYPE=pc KEYTABLE=us rd_NO_DM rhgb quiet
	initrd /initramfs-2.6.32-642.el6.x86_64.img
"""  # noqa


GRUB1_OUTPUTS = [
    # noqa
    # No problem.
    (
        {'kernel_boot_options': ''},
        {},
    ),
    # Problematic.
    (
        {'kernel_boot_options': 'selinux=0'},
        {GRUB_DISABLED: [
            '/vmlinuz-2.6.32-642.el6.x86_64 selinux=0 ro root=/dev/mapper/VolGroup-lv_root rd_NO_LUKS LANG=en_US.UTF-8 rd_NO_MD rd_LVM_LV=VolGroup/lv_swap SYSFONT=latarcyrheb-sun16 crashkernel=auto rd_LVM_LV=VolGroup/lv_root  KEYBOARDTYPE=pc KEYTABLE=us rd_NO_DM rhgb quiet',
        ]},
    ),
    (
        {'kernel_boot_options': 'enforcing=0'},
        {GRUB_NOT_ENFORCING: [
            '/vmlinuz-2.6.32-642.el6.x86_64 enforcing=0 ro root=/dev/mapper/VolGroup-lv_root rd_NO_LUKS LANG=en_US.UTF-8 rd_NO_MD rd_LVM_LV=VolGroup/lv_swap SYSFONT=latarcyrheb-sun16 crashkernel=auto rd_LVM_LV=VolGroup/lv_root  KEYBOARDTYPE=pc KEYTABLE=us rd_NO_DM rhgb quiet',
        ]},
    ),
    (
        {'kernel_boot_options': 'selinux=0 enforcing=0'},
        {
            GRUB_DISABLED: [
                '/vmlinuz-2.6.32-642.el6.x86_64 selinux=0 enforcing=0 ro root=/dev/mapper/VolGroup-lv_root rd_NO_LUKS LANG=en_US.UTF-8 rd_NO_MD rd_LVM_LV=VolGroup/lv_swap SYSFONT=latarcyrheb-sun16 crashkernel=auto rd_LVM_LV=VolGroup/lv_root  KEYBOARDTYPE=pc KEYTABLE=us rd_NO_DM rhgb quiet',
            ],
            GRUB_NOT_ENFORCING: [
                '/vmlinuz-2.6.32-642.el6.x86_64 selinux=0 enforcing=0 ro root=/dev/mapper/VolGroup-lv_root rd_NO_LUKS LANG=en_US.UTF-8 rd_NO_MD rd_LVM_LV=VolGroup/lv_swap SYSFONT=latarcyrheb-sun16 crashkernel=auto rd_LVM_LV=VolGroup/lv_root  KEYBOARDTYPE=pc KEYTABLE=us rd_NO_DM rhgb quiet',
            ]
        },
    ),
]

# rhel-7
GRUB2_TEMPLATE = """
#
# DO NOT EDIT THIS FILE
#
# It is automatically generated by grub2-mkconfig using templates
# from /etc/grub.d and settings from /etc/default/grub
#

### BEGIN /etc/grub.d/00_header ###
set pager=1

if [ -s $prefix/grubenv ]; then
  load_env
fi
if [ "${next_entry}" ] ; then
   set default="${next_entry}"
   set next_entry=
   save_env next_entry
   set boot_once=true
else
   set default="${saved_entry}"
fi

if [ x"${feature_menuentry_id}" = xy ]; then
  menuentry_id_option="--id"
else
  menuentry_id_option=""
fi

export menuentry_id_option

if [ "${prev_saved_entry}" ]; then
  set saved_entry="${prev_saved_entry}"
  save_env saved_entry
  set prev_saved_entry=
  save_env prev_saved_entry
  set boot_once=true
fi

function savedefault {
  if [ -z "${boot_once}" ]; then
    saved_entry="${chosen}"
    save_env saved_entry
  fi
}

function load_video {
  if [ x$feature_all_video_module = xy ]; then
    insmod all_video
  else
    insmod efi_gop
    insmod efi_uga
    insmod ieee1275_fb
    insmod vbe
    insmod vga
    insmod video_bochs
    insmod video_cirrus
  fi
}

terminal_output console
if [ x$feature_timeout_style = xy ] ; then
  set timeout_style=menu
  set timeout=5
# Fallback normal timeout code in case the timeout_style feature is
# unavailable.
else
  set timeout=5
fi
### END /etc/grub.d/00_header ###

### BEGIN /etc/grub.d/00_tuned ###
set tuned_params=""
### END /etc/grub.d/00_tuned ###

### BEGIN /etc/grub.d/01_users ###
if [ -f ${prefix}/user.cfg ]; then
  source ${prefix}/user.cfg
  if [ -n ${GRUB2_PASSWORD} ]; then
    set superusers="root"
    export superusers
    password_pbkdf2 root ${GRUB2_PASSWORD}
  fi
fi
### END /etc/grub.d/01_users ###

### BEGIN /etc/grub.d/10_linux ###
menuentry 'Red Hat Enterprise Linux Server (3.10.0-327.el7.x86_64) 7.2 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-327.el7.x86_64-advanced-4f80b3d4-90ba-4545-869c-febdecc586ce' {
	load_video
	set gfxpayload=keep
	insmod gzio
	insmod part_msdos
	insmod xfs
	set root='hd0,msdos1'
	if [ x$feature_platform_search_hint = xy ]; then
	  search --no-floppy --fs-uuid --set=root --hint-bios=hd0,msdos1 --hint-efi=hd0,msdos1 --hint-baremetal=ahci0,msdos1 --hint='hd0,msdos1'  860a7b56-dbdd-498a-b085-53dc93e4650b
	else
	  search --no-floppy --fs-uuid --set=root 860a7b56-dbdd-498a-b085-53dc93e4650b
	fi
	linux16 /vmlinuz-3.10.0-327.el7.x86_64 %s root=/dev/mapper/rhel-root ro crashkernel=auto rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet LANG=en_US.UTF-8
	initrd16 /initramfs-3.10.0-327.el7.x86_64.img
}
menuentry 'Red Hat Enterprise Linux Server (0-rescue-9f20b35c9faa49aebe171f62a11b236f) 7.2 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-0-rescue-9f20b35c9faa49aebe171f62a11b236f-advanced-4f80b3d4-90ba-4545-869c-febdecc586ce' {
	load_video
	insmod gzio
	insmod part_msdos
	insmod xfs
	set root='hd0,msdos1'
	if [ x$feature_platform_search_hint = xy ]; then
	  search --no-floppy --fs-uuid --set=root --hint-bios=hd0,msdos1 --hint-efi=hd0,msdos1 --hint-baremetal=ahci0,msdos1 --hint='hd0,msdos1'  860a7b56-dbdd-498a-b085-53dc93e4650b
	else
	  search --no-floppy --fs-uuid --set=root 860a7b56-dbdd-498a-b085-53dc93e4650b
	fi
	linux16 /vmlinuz-0-rescue-9f20b35c9faa49aebe171f62a11b236f %s root=/dev/mapper/rhel-root ro crashkernel=auto rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet
	initrd16 /initramfs-0-rescue-9f20b35c9faa49aebe171f62a11b236f.img
}

### END /etc/grub.d/10_linux ###

### BEGIN /etc/grub.d/20_linux_xen ###
### END /etc/grub.d/20_linux_xen ###

### BEGIN /etc/grub.d/20_ppc_terminfo ###
### END /etc/grub.d/20_ppc_terminfo ###

### BEGIN /etc/grub.d/30_os-prober ###
### END /etc/grub.d/30_os-prober ###

### BEGIN /etc/grub.d/40_custom ###
# This file provides an easy way to add custom menu entries.  Simply type the
# menu entries you want to add after this comment.  Be careful not to change
# the 'exec tail' line above.
### END /etc/grub.d/40_custom ###

### BEGIN /etc/grub.d/41_custom ###
if [ -f  ${config_directory}/custom.cfg ]; then
  source ${config_directory}/custom.cfg
elif [ -z "${config_directory}" -a -f  $prefix/custom.cfg ]; then
  source $prefix/custom.cfg;
fi
### END /etc/grub.d/41_custom ###
"""  # noqa

GRUB2_OUTPUTS = [
    # No problem.
    (
        {'kernel_boot_options': ''},
        {},
    ),
    # Problematic.
    (
        {'kernel_boot_options': 'selinux=0'},
        {GRUB_DISABLED: [
            '/vmlinuz-3.10.0-327.el7.x86_64 selinux=0 root=/dev/mapper/rhel-root ro crashkernel=auto rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet LANG=en_US.UTF-8',
            '/vmlinuz-0-rescue-9f20b35c9faa49aebe171f62a11b236f selinux=0 root=/dev/mapper/rhel-root ro crashkernel=auto rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet',
        ]},
    ),
    (
        {'kernel_boot_options': 'enforcing=0'},
        {GRUB_NOT_ENFORCING: [
            '/vmlinuz-3.10.0-327.el7.x86_64 enforcing=0 root=/dev/mapper/rhel-root ro crashkernel=auto rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet LANG=en_US.UTF-8',
            '/vmlinuz-0-rescue-9f20b35c9faa49aebe171f62a11b236f enforcing=0 root=/dev/mapper/rhel-root ro crashkernel=auto rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet',
        ]},
    ),
    (
        {'kernel_boot_options': 'selinux=0 enforcing=0'},
        {
            GRUB_DISABLED: [
                '/vmlinuz-3.10.0-327.el7.x86_64 selinux=0 enforcing=0 root=/dev/mapper/rhel-root ro crashkernel=auto rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet LANG=en_US.UTF-8',
                '/vmlinuz-0-rescue-9f20b35c9faa49aebe171f62a11b236f selinux=0 enforcing=0 root=/dev/mapper/rhel-root ro crashkernel=auto rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet',
            ],
            GRUB_NOT_ENFORCING: [
                '/vmlinuz-3.10.0-327.el7.x86_64 selinux=0 enforcing=0 root=/dev/mapper/rhel-root ro crashkernel=auto rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet LANG=en_US.UTF-8',
                '/vmlinuz-0-rescue-9f20b35c9faa49aebe171f62a11b236f selinux=0 enforcing=0 root=/dev/mapper/rhel-root ro crashkernel=auto rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet',
            ]
        },
    ),
]

TEST_CASES_1 = [
    ((SESTATUS_OUT, SELINUX_CONF, GRUB1_TEMPLATE),
     (True, {})),
    ((SESTATUS_OUT, SELINUX_CONF, GRUB1_TEMPLATE.format(kernel_boot_options='selinux=0')),
     (False, {GRUB_DISABLED: ['/vmlinuz-2.6.32-642.el6.x86_64 selinux=0 ro root=/dev/mapper/VolGroup-lv_root rd_NO_LUKS LANG=en_US.UTF-8 rd_NO_MD rd_LVM_LV=VolGroup/lv_swap SYSFONT=latarcyrheb-sun16 crashkernel=auto rd_LVM_LV=VolGroup/lv_root  KEYBOARDTYPE=pc KEYTABLE=us rd_NO_DM rhgb quiet']})),
    ((SESTATUS_OUT, SELINUX_CONF, GRUB1_TEMPLATE.format(kernel_boot_options='enforcing=0')),
     (False, {GRUB_NOT_ENFORCING: ['/vmlinuz-2.6.32-642.el6.x86_64 enforcing=0 ro root=/dev/mapper/VolGroup-lv_root rd_NO_LUKS LANG=en_US.UTF-8 rd_NO_MD rd_LVM_LV=VolGroup/lv_swap SYSFONT=latarcyrheb-sun16 crashkernel=auto rd_LVM_LV=VolGroup/lv_root  KEYBOARDTYPE=pc KEYTABLE=us rd_NO_DM rhgb quiet']})),
    ((SESTATUS_OUT, SELINUX_CONF_DISABLED, GRUB1_TEMPLATE),
     (False, {BOOT_DISABLED: 'disabled'})),
    ((SESTATUS_OUT, SELINUX_CONF_NOT_ENFORCING, GRUB1_TEMPLATE),
     (False, {BOOT_NOT_ENFORCING: 'permissive'})),
    ((SESTATUS_OUT, SELINUX_CONF_COMMENTED, GRUB1_TEMPLATE),
     (False, {BOOT_NOT_ENFORCING: 'Missing in config (Permissive by default)'})),
    ((SESTATUS_OUT_DISABLED, SELINUX_CONF_NOT_ENFORCING, GRUB1_TEMPLATE),
     (False, {RUNTIME_DISABLED: 'disabled', BOOT_NOT_ENFORCING: 'permissive'})),
    ((SESTATUS_OUT_NOT_ENFORCING, SELINUX_CONF_DISABLED, GRUB1_TEMPLATE.format(kernel_boot_options='selinux=0')),
     (False, {GRUB_DISABLED: ['/vmlinuz-2.6.32-642.el6.x86_64 selinux=0 ro root=/dev/mapper/VolGroup-lv_root rd_NO_LUKS LANG=en_US.UTF-8 rd_NO_MD rd_LVM_LV=VolGroup/lv_swap SYSFONT=latarcyrheb-sun16 crashkernel=auto rd_LVM_LV=VolGroup/lv_root  KEYBOARDTYPE=pc KEYTABLE=us rd_NO_DM rhgb quiet'],
              RUNTIME_NOT_ENFORCING: 'permissive',
              BOOT_DISABLED: 'disabled'
              })),
]
TEST_CASES_2 = [
    ((SESTATUS_OUT, SELINUX_CONF, GRUB2_TEMPLATE % ('selinux=0', 'selinux=0')),
     (False, {GRUB_DISABLED: ['/vmlinuz-3.10.0-327.el7.x86_64 selinux=0 root=/dev/mapper/rhel-root ro crashkernel=auto rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet LANG=en_US.UTF-8',
                              '/vmlinuz-0-rescue-9f20b35c9faa49aebe171f62a11b236f selinux=0 root=/dev/mapper/rhel-root ro crashkernel=auto rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet',
                              ]})),
    ((SESTATUS_OUT_DISABLED, SELINUX_CONF, GRUB2_TEMPLATE),
     (False, {RUNTIME_DISABLED: 'disabled'})),
    ((SESTATUS_OUT_NOT_ENFORCING, SELINUX_CONF, GRUB2_TEMPLATE),
     (False, {RUNTIME_NOT_ENFORCING: 'permissive'})),
]


def test_integration():
    import pprint
    for inputs, outputs in TEST_CASES_1:
        sestatus = SEStatus(context_wrap(inputs[0]))
        selinux_config = SelinuxConfig(context_wrap(inputs[1]))
        grub_config = Grub1Config(context_wrap(inputs[2]))
        grub_conf = GrubConf(grub_config, None, None, None, None, None, None, None)
        selinux = SELinux(sestatus, selinux_config, grub_conf)
        assert selinux.ok() == outputs[0]
        assert selinux.problems == outputs[1]
        pprint.pprint(selinux.problems)

    for inputs, outputs in TEST_CASES_2:
        sestatus = SEStatus(context_wrap(inputs[0]))
        selinux_config = SelinuxConfig(context_wrap(inputs[1]))
        grub_config = Grub2Config(context_wrap(inputs[2]))
        grub_conf = GrubConf(None, grub_config, None, None, None, None, None, None)
        selinux = SELinux(sestatus, selinux_config, grub_conf)
        assert selinux.ok() == outputs[0]
        assert selinux.problems == outputs[1]
        pprint.pprint(selinux.problems)



import doctest
import pytest

from insights.core.exceptions import ContentException, ParseException, SkipComponent
from insights.parsers import ssl_certificate
from insights.tests import context_wrap


CERTIFICATE_OUTPUT1 = """
issuer= /C=US/ST=North Carolina/L=Raleigh/O=Katello/OU=SomeOrgUnit/CN=a.b.c.com
notBefore=Dec  7 07:02:33 2020 GMT
notAfter=Jan 18 07:02:33 2038 GMT
subject= /C=US/ST=North Carolina/L=Raleigh/O=Katello/OU=SomeOrgUnit/CN=a.b.c.com
"""

CERTIFICATE_CHAIN_OUTPUT1 = """
issuer= /C=US/ST=North Carolina/L=Raleigh/O=Katello/OU=SomeOrgUnit/CN=test.a.com
subject= /C=US/ST=North Carolina/L=Raleigh/O=Katello/OU=SomeOrgUnit/CN=test.b.com
notBefore=Dec  7 07:02:33 2020 GMT
notAfter=Jan 18 07:02:33 2038 GMT


issuer= /C=US/ST=North Carolina/L=Raleigh/O=Katello/OU=SomeOrgUnit/CN=test.d.com
subject= /C=US/ST=North Carolina/O=Katello/OU=SomeOrgUnit/CN=test.c.com
notBefore=Nov 30 07:02:42 2020 GMT
notAfter=Jan 18 07:02:43 2018 GMT
"""

CERTIFICATE_CHAIN_OUTPUT2 = """
notAfter=Dec  4 07:04:05 2035 GMT
subject= /CN=Puppet CA: abc.d.com
issuer= /C=US/ST=North Carolina/L=Raleigh/O=Katello/OU=SomeOrgUnit/CN=abc.d.com
"""

SATELLITE_OUTPUT1 = """
subject= /C=US/ST=North Carolina/L=Raleigh/O=Katello/OU=SomeOrgUnit/CN=test.a.com
notAfter=Jan 18 07:02:33 2038 GMT

subject= /C=US/ST=North Carolina/O=Katello/OU=SomeOrgUnit/CN=test.b.com
notAfter=Jan 18 07:02:43 2018 GMT

subject= /C=US/ST=North Carolina/O=Katello/OU=SomeOrgUnit/CN=test.c.com
notAfter=Jan 18 07:02:43 2048 GMT
"""

SATELLITE_OUTPUT2 = """
subject= /C=US/ST=North Carolina/L=Raleigh/O=Katello/OU=SomeOrgUnit/CN=test.a.com
notAfter=Jan 18 07:02:33 2038 GMT

subject= /C=US/ST=North Carolina/O=Katello/OU=SomeOrgUnit/CN=test.b.com
notAfter=Jan 18 07:02:43 2028 GMT
"""

RHSM_KATELLO_CERT_OUTPUT1 = """
issuer= /C=US/ST=North Carolina/L=Raleigh/O=Katello/OU=SomeOrgUnit/CN=a.b.c.com
"""

BAD_OUTPUT1 = """
subject= /C=US/ST=North Carolina/L=Raleigh/O=Katello/OU=SomeOrgUnit/CN=test.a.com
notAfterJan 18 07:02:33 2038 GMT
"""

BAD_OUTPUT2 = """
subject= /C=US/ST=North Carolina/L=Raleigh/O=Katello/OU=SomeOrgUnit/CN=test.a.com
notAfter=2038 Jan 18 07:02:33 GMT
"""

BAD_OUTPUT3 = """
Error opening Certificate /etc/rhsm/ca/katello-default-ca.pem
139814540982160:error:02001002:system library:fopen:No such file or directory:bss_file.c:402:fopen('/etc/rhsm/ca/katello-default-ca.pem','r')
139814540982160:error:20074002:BIO routines:FILE_CTRL:system lib:bss_file.c:404:
unable to load certificate
"""

BAD_OUTPUT4 = """

"""

HTTPD_CERT_EXPIRE_INFO = '''
notAfter=Jan 18 07:02:43 2038 GMT
'''

NGINX_CERT_EXPIRE_INFO = '''
notAfter=Jan 18 07:02:43 2038 GMT
'''

MSSQL_CERT_EXPIRE_INFO = '''
notAfter=Nov  5 01:43:59 2022 GMT
'''

NSS_CERT_OUTPUT = """
        Not After : Sun Dec 07 05:26:10 2025"""

NSS_CERT_BAD_OUTPUT_1 = """
            Not After : Dec 07 05:26:10 2025"""

NSS_CERT_BAD_OUTPUT_2 = """
        Issuer: "CN=Certificate Shack,O=huali.node2.redhat.com,C=CN
""".strip()


def test_certificate_info_exception():
    with pytest.raises(ParseException):
        ssl_certificate.CertificateInfo(context_wrap(BAD_OUTPUT1))
    with pytest.raises(ParseException):
        ssl_certificate.CertificateInfo(context_wrap(BAD_OUTPUT2))
    with pytest.raises(ContentException):
        ssl_certificate.CertificateInfo(context_wrap(BAD_OUTPUT3))
    with pytest.raises(SkipComponent):
        ssl_certificate.CertificateInfo(context_wrap(BAD_OUTPUT4))


def test_certificate_chain_exception():
    with pytest.raises(SkipComponent):
        ssl_certificate.CertificateChain(context_wrap(BAD_OUTPUT4))


def test_certificate_info():
    cert = ssl_certificate.CertificateInfo(context_wrap(CERTIFICATE_OUTPUT1, args='/a/b/c.pem'))
    assert cert['issuer'] == '/C=US/ST=North Carolina/L=Raleigh/O=Katello/OU=SomeOrgUnit/CN=a.b.c.com'
    assert cert['notBefore'].str == 'Dec  7 07:02:33 2020'
    assert cert['notAfter'].str == 'Jan 18 07:02:33 2038'
    assert cert['subject'] == '/C=US/ST=North Carolina/L=Raleigh/O=Katello/OU=SomeOrgUnit/CN=a.b.c.com'
    assert cert.cert_path == '/a/b/c.pem'


def test_certificates_chain():
    certs = ssl_certificate.CertificateChain(context_wrap(CERTIFICATE_CHAIN_OUTPUT1))
    assert len(certs) == 2
    assert certs.earliest_expiry_date.str == 'Jan 18 07:02:43 2018'
    for cert in certs:
        if cert['notAfter'].str == certs.earliest_expiry_date.str:
            assert cert['issuer'] == '/C=US/ST=North Carolina/L=Raleigh/O=Katello/OU=SomeOrgUnit/CN=test.d.com'
            assert cert['notBefore'].str == 'Nov 30 07:02:42 2020'
            assert cert['subject'] == '/C=US/ST=North Carolina/O=Katello/OU=SomeOrgUnit/CN=test.c.com'
            assert cert['notBefore'].str == 'Nov 30 07:02:42 2020'

    certs = ssl_certificate.CertificateChain(context_wrap(CERTIFICATE_CHAIN_OUTPUT2))
    assert len(certs) == 1
    assert certs[0]['issuer'] == '/C=US/ST=North Carolina/L=Raleigh/O=Katello/OU=SomeOrgUnit/CN=abc.d.com'

    certs = ssl_certificate.CertificateChain(context_wrap(RHSM_KATELLO_CERT_OUTPUT1))
    assert len(certs) == 1


def test_satellite_ca_chain():
    certs = ssl_certificate.SatelliteCustomCaChain(context_wrap(SATELLITE_OUTPUT1))
    assert len(certs) == 3
    assert certs.earliest_expiry_date.str == 'Jan 18 07:02:43 2018'
    assert certs[0]['subject'] == '/C=US/ST=North Carolina/L=Raleigh/O=Katello/OU=SomeOrgUnit/CN=test.a.com'
    assert certs[0]['notAfter'].str == 'Jan 18 07:02:33 2038'
    assert certs[1]['subject'] == '/C=US/ST=North Carolina/O=Katello/OU=SomeOrgUnit/CN=test.b.com'
    assert certs[1]['notAfter'].str == 'Jan 18 07:02:43 2018'
    assert certs[2]['subject'] == '/C=US/ST=North Carolina/O=Katello/OU=SomeOrgUnit/CN=test.c.com'
    assert certs[2]['notAfter'].str == 'Jan 18 07:02:43 2048'


def test_rhsm_katello_default_ca():
    rhsm_katello_default_ca = ssl_certificate.RhsmKatelloDefaultCACert(context_wrap(RHSM_KATELLO_CERT_OUTPUT1))
    assert rhsm_katello_default_ca['issuer'] == '/C=US/ST=North Carolina/L=Raleigh/O=Katello/OU=SomeOrgUnit/CN=a.b.c.com'


def test_doc():
    cert = ssl_certificate.CertificateInfo(context_wrap(CERTIFICATE_OUTPUT1))
    ca_cert = ssl_certificate.CertificateChain(context_wrap(CERTIFICATE_CHAIN_OUTPUT1))
    satellite_ca_certs = ssl_certificate.SatelliteCustomCaChain(context_wrap(SATELLITE_OUTPUT2))
    rhsm_katello_default_ca = ssl_certificate.RhsmKatelloDefaultCACert(context_wrap(RHSM_KATELLO_CERT_OUTPUT1))
    date_info = ssl_certificate.HttpdSSLCertExpireDate(context_wrap(HTTPD_CERT_EXPIRE_INFO))
    nginx_date_info = ssl_certificate.NginxSSLCertExpireDate(context_wrap(HTTPD_CERT_EXPIRE_INFO, args='/a/b/c.pem'))
    mssql_date_info = ssl_certificate.MssqlTLSCertExpireDate(context_wrap(MSSQL_CERT_EXPIRE_INFO))
    cert_info = ssl_certificate.HttpdCertInfoInNSS(context_wrap(NSS_CERT_OUTPUT))
    globs = {
        'cert': cert,
        'certs': ca_cert,
        'satellite_ca_certs': satellite_ca_certs,
        'rhsm_katello_default_ca': rhsm_katello_default_ca,
        'date_info': date_info,
        'nginx_date_info': nginx_date_info,
        'mssql_date_info': mssql_date_info,
        'nss_cert_info': cert_info
    }
    failed, _ = doctest.testmod(ssl_certificate, globs=globs)
    assert failed == 0


def test_httpd_ssl_cert_parser():
    date_info = ssl_certificate.HttpdSSLCertExpireDate(context_wrap(HTTPD_CERT_EXPIRE_INFO))
    assert 'notAfter' in date_info
    assert date_info['notAfter'].str == 'Jan 18 07:02:43 2038'


def test_nginx_ssl_cert_parser():
    date_info = ssl_certificate.NginxSSLCertExpireDate(context_wrap(NGINX_CERT_EXPIRE_INFO, args='/test/c.pem'))
    assert 'notAfter' in date_info
    assert date_info['notAfter'].str == 'Jan 18 07:02:43 2038'
    assert date_info.cert_path == '/test/c.pem'


def test_mssql_tls_cert_parser():
    date_info = ssl_certificate.MssqlTLSCertExpireDate(context_wrap(MSSQL_CERT_EXPIRE_INFO))
    assert 'notAfter' in date_info
    assert date_info['notAfter'].str == 'Nov  5 01:43:59 2022'


def test_httpd_cert_info_in_nss():
    cert_info = ssl_certificate.HttpdCertInfoInNSS(context_wrap(NSS_CERT_OUTPUT))
    assert 'notAfter' in cert_info
    assert cert_info['notAfter'].str == 'Sun Dec 07 05:26:10 2025'


def test_httpd_cert_info_in_nss_exception():
    with pytest.raises(ParseException):
        ssl_certificate.HttpdCertInfoInNSS(context_wrap(NSS_CERT_BAD_OUTPUT_1))

    with pytest.raises(SkipComponent):
        ssl_certificate.HttpdCertInfoInNSS(context_wrap(NSS_CERT_BAD_OUTPUT_2))



from insights.core import JSONParser
from insights.core.plugins import parser
from insights.specs import Specs
from insights.util import deprecated


@parser(Specs.tags)
class Tags(JSONParser):
    """
    .. warning::
        This parser is deprecated, please use
        :py:class:`insights.parsers.client_metadata.Tags` instead.

    Class for parsing the content of ``tags.json``."""
    def __init__(self, *args, **kwargs):
        deprecated(
            Tags,
            "Please use insights.parsers.client_metadata.Tags instead.",
            "3.4.0"
        )
        super(Tags, self).__init__(*args, **kwargs)



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import mpirun
from insights.parsers.mpirun import MPIrunVersion
from insights.tests import context_wrap

MPIRUN_VERSION_1 = """
Intel(R) MPI Library for Linux* OS, Version 2019 Update 08 Build 202010429 (id: e380127cb)
Copyright 2003-2020, Intel Corporation.
""".strip()

MPIRUN_VERSION_2 = """
Intel(R) MPI Library for Linux* OS, Version 2021 Update 15 Build 20200312 (id: 5dc2dd3e9)
Copyright 2003-2022, Intel Corporation.
""".strip()

MPIRUN_VERSION_3 = """
-bash: /usr/local/bin/mpirun: No Version file or directory
""".strip()

MPIRUN_VERSION_4 = ""


def test_mpirun_version():
    ret = MPIrunVersion(context_wrap(MPIRUN_VERSION_1))
    assert ret.year == '2019'
    assert ret.version == "Version 2019 Update 08 Build 202010429 (id: e380127cb)"

    ret = MPIrunVersion(context_wrap(MPIRUN_VERSION_2))
    assert ret.year == '2021'
    assert ret.version == "Version 2021 Update 15 Build 20200312 (id: 5dc2dd3e9)"


def test_mpirun_version_ab():
    with pytest.raises(SkipComponent) as e_skip:
        MPIrunVersion(context_wrap(MPIRUN_VERSION_4))
    assert "Empty content" in str(e_skip.value)

    with pytest.raises(SkipComponent) as e_skip:
        MPIrunVersion(context_wrap(MPIRUN_VERSION_3))
    assert "Content not parsable" in str(e_skip.value)


def test_doc_examples():
    env = {
        'mpirun_ver': MPIrunVersion(context_wrap(MPIRUN_VERSION_1))
    }
    failed, total = doctest.testmod(mpirun, globs=env)
    assert failed == 0
    assert total == 2



import string
from insights.parsr import (EOF, Forward, LeftCurly, Lift, LineEnd, RightCurly,
        Many, Number, OneLineComment, PosMarker, SemiColon, QuotedString,
        skip_none, String, WS, WSChar)
from insights.parsr.query import Entry


def loads(data):
    return Top(data)[0]


def load(f):
    return loads(f.read())


def to_entry(name, attrs, body):
    if body == ";":
        return Entry(name=name.value, attrs=attrs, lineno=name.lineno)
    return Entry(name=name.value, attrs=attrs, children=body, lineno=name.lineno)


Stmt = Forward()
Num = Number & (WSChar | LineEnd)
Comment = OneLineComment("#").map(lambda x: None)
BeginBlock = WS >> LeftCurly << WS
EndBlock = WS >> RightCurly << WS
Bare = String(set(string.printable) - (set(string.whitespace) | set("#;{}'\"")))
Name = WS >> PosMarker(String(string.ascii_letters + "_")) << WS
Attr = WS >> (Num | Bare | QuotedString) << WS
Attrs = Many(Attr)
Block = BeginBlock >> Many(Stmt).map(skip_none) << EndBlock
Stanza = (Lift(to_entry) * Name * Attrs * (Block | SemiColon)) | Comment
Stmt <= WS >> Stanza << WS
Doc = Many(Stmt).map(skip_none)
Top = Doc + EOF



"""
LsVarLibRsyslog - command ``ls -lZ  /var/lib/rsyslog``
======================================================
"""


from insights.core import CommandParser, FileListing
from insights.core.plugins import parser
from insights.specs import Specs
from insights.util import deprecated


@parser(Specs.ls_var_lib_rsyslog)
class LsVarLibRsyslog(CommandParser, FileListing):
    """
    .. warning::
        This class is deprecated and will be removed from 3.5.0.
        Please use the :class:`insights.parsers.ls.LSlanZ` instead.

    Parses output of ``ls -lZ  /var/lib/rsyslog`` command.

    Sample output::

        total 4
        -rw-------. 1 root root system_u:object_r:syslogd_var_lib_t:s0 127 Nov 30 03:40 imjournal.state

    Examples:
        >>> rsyslog_obj.dir_contains('/var/lib/rsyslog', 'imjournal.state')
        True
        >>> imjournal_entry = rsyslog_obj.dir_entry('/var/lib/rsyslog', 'imjournal.state')
        >>> imjournal_entry['se_type']
        'syslogd_var_lib_t'
    """
    def __init__(self, *args, **kwargs):
        deprecated(LsVarLibRsyslog, "Please use the :class:`insights.parsers.ls.LSlanZ` instead.", "3.5.0")
        super(LsVarLibRsyslog, self).__init__(*args, **kwargs)



"""
AzureInstance
=============

The following Azure Instance related parsers are placed in this module:

AzureInstanceID - 'vmId' of Azure Instance
------------------------------------------

AzureInstanceType - 'vmSize' of Azure Instance
----------------------------------------------

AzureInstancePlan - 'plan' of Azure Instance
--------------------------------------------

AzurePublicIpv4Addresses - list of public IPv4 addresses
--------------------------------------------------------
"""
import json

from insights.core import CommandParser
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


def validate_content(content):
    if not content or 'curl: ' in content[0]:
        raise SkipComponent()


@parser(Specs.azure_instance_id)
class AzureInstanceID(CommandParser):
    """
    Class for parsing the Azure Instance type returned by command
    ``curl -s -H Metadata:true http://169.254.169.254/metadata/instance/compute/vmId?api-version=2021-12-13&format=text``,


    Typical output of this command is::

        f904ece8-c6c1-4b5c-881f-309b50f25e50

    Raises:
        SkipComponent: When content is empty or no parse-able content.

    Attributes:
        id (str): The instance ID of the VM instance in Azure.

    Examples:
        >>> azure_id.id
        'f904ece8-c6c1-4b5c-881f-309b50f25e50'
    """

    def parse_content(self, content):
        validate_content(content)

        self.id = content[0].strip()

    def __repr__(self):
        return "<instance_id: {i}>".format(i=self.id)


@parser(Specs.azure_instance_type)
class AzureInstanceType(CommandParser):
    """
    Class for parsing the Azure Instance type returned by command
    ``curl -s -H Metadata:true http://169.254.169.254/metadata/instance/compute/vmSize?api-version=2021-12-13&format=text``,


    Typical output of this command is::

        Standard_L64s_v2

    Raises:
        SkipComponent: When content is empty or no parse-able content.
        ParseException: When type cannot be recognized.

    Attributes:
        type (str): The type of VM instance in Azure, e.g: Standard
        size (str): The size of VM instance in Azure, e.g: L64s, NC12s
        version (str): The version of VM instance in Azure, e.g: v2, v3, `None` for non-version
        raw (str): The fully type string returned by the ``curl`` command

    Examples:
        >>> azure_type.type
        'Standard'
        >>> azure_type.size
        'L64s'
        >>> azure_type.version
        'v2'
        >>> azure_type.raw
        'Standard_L64s_v2'
    """
    def parse_content(self, content):
        validate_content(content)

        self.raw = self.type = self.size = self.version = None
        # Ignore any curl stats that may be present in data
        for l in content:
            l_strip = l.strip()
            if ' ' not in l_strip and '_' in l_strip:
                self.raw = l_strip
                type_sp = l_strip.split('_')
                self.type, self.size = type_sp[0], type_sp[1]
                if len(type_sp) >= 3:
                    self.version = type_sp[2]

        if not self.type:
            raise ParseException('Unrecognized type: "{0}"', content[0])

    def __repr__(self):
        return "<azure_type: {t}, size: {s}, version: {v},  raw: {r}>".format(
                t=self.type, s=self.size, v=self.version, r=self.raw)


@parser(Specs.azure_instance_plan)
class AzureInstancePlan(CommandParser):
    """
    Class for parsing the Azure Instance Plan returned by command
    ``curl -s -H Metadata:true http://169.254.169.254/metadata/instance/compute/plan?api-version=2021-12-13&format=json``,


    Typical Output of this command is::

        {
            "name": "planName",
            "product": "planProduct",
            "publisher": "planPublisher"
        },

    Raises:
        SkipComponent: When content is empty or no parse-able content.

    Attributes:
        name (str): The name of the plan for the VM Instance in Azure, e.g: rhel7
        product (str): The product of the plan for the VM Instance in Azure, e.g: RHEL
        publisher (str): The publisher of the plan for the VM Instance in Azure, e.g: Red hat
        raw (str): The full JSON of the plan returned by the ``curl`` command

    Examples:
        >>> azure_plan.name == 'planName'
        True
        >>> azure_plan.product == 'planProduct'
        True
        >>> azure_plan.publisher == 'planPublisher'
        True
    """
    def parse_content(self, content):
        validate_content(content)

        try:
            plan = json.loads(content[0])
        except:
            raise ParseException("Unable to parse JSON")

        self.raw = content[0]
        self.name = plan["name"] if plan["name"] != "" else None
        self.product = plan["product"] if plan["product"] != "" else None
        self.publisher = plan["publisher"] if plan["publisher"] != "" else None

    def __repr__(self):
        return "<azure_plan_name: {n}, product: {pr}, publisher: {pu}, raw: {r}".format(
            n=self.name, pr=self.product, pu=self.publisher, r=self.raw
        )


@parser(Specs.azure_load_balancer)
class AzurePublicIpv4Addresses(CommandParser, list):
    """
    Class for parsing the Azure load balancer JSON data returned by command

        curl -sH "Metadata:true" --connect-timeout 5 \
            "http://169.254.169.254/metadata/loadbalancer?api-version=2021-12-13&format=json"

    Typical Output of this command is::

        {
          "loadbalancer": {
            "publicIpAddresses": [
              {
                "frontendIpAddress": "137.116.118.209",
                "privateIpAddress": "10.0.0.4"
              }
            ],
            "inboundRules": [],
            "outboundRules": []
          }
        }

    Raises:
        SkipComponent: When content is empty or curl returned an error.
        ParseException: On JSON parsing error.
    """
    def parse_content(self, content):
        validate_content(content)

        try:
            plan = json.loads(' '.join(content))
            for pair in plan["loadbalancer"]["publicIpAddresses"]:
                self.append(pair["frontendIpAddress"])
        except:
            raise ParseException("Unable to parse JSON")



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import iris
from insights.tests import context_wrap


IRIS_RUNNING = """

Configuration 'IRIS'   (default)
    directory:    /intersystems
    versionid:    2023.1.0.235.1com
    datadir:      /intersystems
    conf file:    iris.cpf  (SuperServer port = 1972, WebServer = 52773)
    status:       running, since Tue Jun 27 01:55:25 2023
    state:        ok
    product:      InterSystems IRIS
""".strip()

IRIS_DOWN = """

Configuration 'IRIS'   (default)
    directory:    /intersystems
    versionid:    2023.1.0.235.1com
    datadir:      /intersystems
    conf file:    iris.cpf  (SuperServer port = 1972, WebServer = 52773)
    status:       down, last used Tue Jun 27 01:50:36 2023
    product:      InterSystems IRIS
""".strip()

IRIS_NO_CONTENT = """"""


def test_iris_list():
    iris_running = iris.IrisList(context_wrap(IRIS_RUNNING))
    assert iris_running['name'] == "IRIS"
    assert iris_running['status'] == "running, since Tue Jun 27 01:55:25 2023"
    assert 'state' in iris_running
    assert iris_running.is_running

    iris_down = iris.IrisList(context_wrap(IRIS_DOWN))
    assert iris_down['name'] == "IRIS"
    assert iris_down['status'] == "down, last used Tue Jun 27 01:50:36 2023"
    assert 'state' not in iris_down
    assert not iris_down.is_running


def test_fail():
    with pytest.raises(SkipComponent) as e:
        iris.IrisList(context_wrap(IRIS_NO_CONTENT))
    assert "The result is empty" in str(e)


def test_doc_examples():
    env = {
        'iris_info': iris.IrisList(
            context_wrap(IRIS_RUNNING)),
    }
    failed, total = doctest.testmod(iris, globs=env)
    assert failed == 0



import doctest

from insights.parsers import ironic_inspector_log
from insights.tests import context_wrap


IRONIC_INSPECTOR_LOG = """
2019-03-01 03:53:02.814 4022 DEBUG futurist.periodics [-] Submitting periodic callback 'ironic_inspector.pxe_filter.base.periodic_sync_task' _process_scheduled /usr/lib/python2.7/site-packages/futurist/periodics.py:639
2019-03-01 03:53:02.815 4022 DEBUG ironic_inspector.pxe_filter.base [-] The PXE filter driver DnsmasqFilter, state=initialized enters the fsm_reset_on_error context fsm_reset_on_error /usr/lib/python2.7/site-packages/ironic_inspector/pxe_filter/base.py:137
2019-03-01 03:53:02.816 4022 DEBUG ironic_inspector.pxe_filter.dnsmasq [-] Syncing the driver _sync /usr/lib/python2.7/site-packages/ironic_inspector/pxe_filter/dnsmasq.py:79
2019-03-01 03:53:02.881 4022 DEBUG ironic_inspector.pxe_filter.dnsmasq [-] The dnsmasq PXE filter was synchronized (took 0:00:00.065231) _sync /usr/lib/python2.7/site-packages/ironic_inspector/pxe_filter/dnsmasq.py:111
2019-03-01 03:53:02.882 4022 DEBUG ironic_inspector.pxe_filter.base [-] The PXE filter driver DnsmasqFilter, state=initialized left the fsm_reset_on_error context fsm_reset_on_error /usr/lib/python2.7/site-packages/ironic_inspector/pxe_filter/base.py:153
""".strip()

IRONIC_INSPECTOR_ERROR_LOG = """
2018-12-05 17:20:41.404 25139 ERROR requests.packages.urllib3.connection [-] Certificate did not match expected hostname: 10.xx.xx.xx. Certificate: {'subjectAltName': (('DNS', '10.xx.xx.xx'),), 'notBefore': u'Dec  4 12:02:36 2018 GMT', 'serialNumber': u'616460101648CCDF5727C', 'notAfter': 'Jun 21 21:40:11 2019 GMT', 'version': 3L, 'subject': ((('commonName', u'10.xx.xx.xx'),),), 'issuer': ((('commonName', u'Local Signing Authority'),), (('commonName', u'616460a1-da41448c-cdf566ff'),))}
""".strip()


def test_ironic_inspector_log():
    log = ironic_inspector_log.IronicInspectorLog(context_wrap(IRONIC_INSPECTOR_LOG))
    assert len(log.lines) == 5


def test_ironic_inspector_error_log():
    log = ironic_inspector_log.IronicInspectorLog(context_wrap(IRONIC_INSPECTOR_ERROR_LOG))
    assert len(log.lines) == 1
    assert log.lines[0] == "2018-12-05 17:20:41.404 25139 ERROR requests.packages.urllib3.connection [-] Certificate did not match expected hostname: 10.xx.xx.xx. Certificate: {'subjectAltName': (('DNS', '10.xx.xx.xx'),), 'notBefore': u'Dec  4 12:02:36 2018 GMT', 'serialNumber': u'616460101648CCDF5727C', 'notAfter': 'Jun 21 21:40:11 2019 GMT', 'version': 3L, 'subject': ((('commonName', u'10.xx.xx.xx'),),), 'issuer': ((('commonName', u'Local Signing Authority'),), (('commonName', u'616460a1-da41448c-cdf566ff'),))}"


def test_ironic_inspector_log_documentation():
    failed_count, tests = doctest.testmod(
        ironic_inspector_log,
        globs={'log': ironic_inspector_log.IronicInspectorLog(context_wrap(IRONIC_INSPECTOR_ERROR_LOG))}
    )
    assert failed_count == 0



import os
import pickle
import time

_KEEPTIME = 300  # 5 minutes


class CacheItem(object):
    def __init__(self, etag, content, cached_at):
        self.etag = etag
        self.content = content
        self.cached_at = cached_at


class URLCache(object):
    """
        URLCache is a simple pickle cache, intended to be used as an HTTP
        response cache.
    """

    def __init__(self, path=None):
        """
            Initialize a URLCache, loading entries from @path, if provided.
        """
        self._path = path
        self._cache = {}
        if os.path.isfile(self._path):
            with open(self._path, "r+b") as f:
                try:
                    self._cache = pickle.load(f)
                except EOFError:
                    self._cache = {}
        if not os.path.exists(os.path.dirname(self._path)):
            os.makedirs(os.path.dirname(self._path))

    def get(self, url):
        try:
            item = self._cache[url]
            if item.cached_at + _KEEPTIME <= time.time():
                del self._cache[url]
                return None
            return self._cache[url]
        except KeyError:
            return None

    def set(self, url, etag, content):
        self._cache[url] = CacheItem(etag, content, time.time())

    def save(self):
        with open(self._path, "w+b") as f:
            pickle.dump(self._cache, f)



from insights.parsers import samba_logs
from insights.parsers.samba_logs import SAMBALog
from insights.tests import context_wrap
import doctest

SAMBA_LOG_NORMAL = """
[2019/11/25 19:29:23.933547,  0] ../lib/util/become_daemon.c:138(daemon_ready)  daemon_ready: STATUS=daemon 'smbd' finished starting up and ready to serve connections
[2019/11/25 19:32:12.511867,  0] ../lib/util/become_daemon.c:138(daemon_ready)  daemon_ready: STATUS=daemon 'smbd' finished starting up and ready to serve connections
[2019/11/27 11:11:04.900434,  5, pid=15817, effective(0, 0), real(0, 0)] ../source3/lib/messages.c:709(messaging_register)  Registering messaging pointer for type 13 - private_data=(nil)
[2019/11/27 11:11:04.900450,  5, pid=15817, effective(0, 0), real(0, 0)] ../source3/lib/messages.c:709(messaging_register)  Registering messaging pointer for type 33 - private_data=0x557a635cb3e0
[2019/11/27 11:11:04.911688, 10, pid=15822, effective(0, 0), real(0, 0)] ../source3/printing/print_cups.c:203(recv_pcap_blob)  successfully recvd blob of len 12
[2019/11/27 11:11:04.911718,  3, pid=15822, effective(0, 0), real(0, 0)] ../source3/printing/print_cups.c:536(cups_async_callback)  failed to retrieve printer list: NT_STATUS_UNSUCCESSFUL
[2019/45/899 11:11:04.911718,  3, pid=15822, effective(0, 0), real(0, 0)] Fake line to check that date parsing handles corrupted dates
[2019/45/899 11:11:04.911891,  3, pid=15822, effective(0, 0), real(0, 0)] ../source3/printing/queue_process.c:236(bq_sig_hup_handler) Reloading pcap cache after SIGHUP.
"""

SAMBALog.keep_scan('sighups', 'SIGHUP')


def test_normal_samba_logs():
    samba_logs = SAMBALog(context_wrap(SAMBA_LOG_NORMAL))
    assert samba_logs is not None
    assert samba_logs.get('messaging_register')[0]['pid'] == 'pid=15817'
    assert samba_logs.get('0x557a635cb3e0')[0]['function'] == '../source3/lib/messages.c:709(messaging_register)'
    assert samba_logs.get('cups_async_callback')[0]['timestamp'] == '2019/11/27 11:11:04.911718'
    assert samba_logs.get('0x557a635cb3e0')[0]['timestamp'] == '2019/11/27 11:11:04.900450'
    assert len(samba_logs.get('Fake line')) == 1


def test_samba_logs_doc():
    env = {"samba_logs": SAMBALog(context_wrap(SAMBA_LOG_NORMAL)), }
    failed, total = doctest.testmod(samba_logs, globs=env)
    assert failed == 0



import pytest

from insights.core.exceptions import ParseException
from insights.parsers.rabbitmq import RabbitMQQueues
from insights.tests import context_wrap

QUEUES = """
cinder-scheduler        0       3       false
cinder-scheduler.ha-controller  0       3       false
cinder-scheduler_fanout_ea9c69fb630f41b2ae6120eba3cd43e0        8141    1   true
cinder-scheduler_fanout_9aed9fbc3d4249289f2cb5ea04c062ab        8145    0   true
cinder-scheduler_fanout_b7a2e488f3ed4e1587b959f9ac255b93        8141    0   true
"""

MORE_QUEUES = """
Listing queues ...
q-agent-notifier-tunnel-delete_fanout_a8dc17bbb6ee469c8aaab31079c01c1c	0	1	false
dhcp_agent.undercloud-per720xd.default.redhat.local	0	1	false
mistral_executor.0.0.0.0	0	1	false
reply_aeff2b095dac4b09b8a4ea27986b402c	0	1	false
reply_6cf2931b774f4a58a14c819570059114	0	1	false
q-agent-notifier-tunnel-update	0	1	false
cert	0	1	false
q-plugin.undercloud-per720xd.default.redhat.local	0	2	false
reply_5d74d09f9758439fbd247097c096c1a2	0	1	false
metering.sample	1	1	false
heat-engine-listener.65218f91-59b6-45bc-99df-63f5af707351	0	1	false
reply_e7fb8381d78b4dcf9501ba8a72ad335f	0	1	false
reply_e366ce8a5b3846608e1e181e9e413231	0	1	false
reply_19ec1f054dc74298b896cbbb06c7b7e7	0	1	false
scheduler.undercloud-per720xd.default.redhat.local	0	1	false
q-server-resource-versions_fanout_56cc075fd8cc486bacd0648225b61970	0	1	false
q-server-resource-versions.undercloud-per720xd.default.redhat.local	0	2	false
mistral_engine.0.0.0.0	0	1	false
reply_c97293591c5c46be9331f0a5662f8961	0	1	false
mistral_engine_fanout_a660b93980ca41ac84b6f1b56cf8a25c	0	1	false
event.sample	0	1	false
...done.
"""

QUEUES_BAD_1 = """
Error: unable to connect to node
"""
QUEUES_BAD_2 = """
queue1 1 x false
"""
QUEUES_BAD_3 = """
queue1 1 1 maybe
"""


def test_rabbitmq_queues():
    queues = RabbitMQQueues(context_wrap(QUEUES))
    assert queues is not None
    assert len(queues.data) == 5
    assert queues.data[0] == RabbitMQQueues.QueueInfo(
        name="cinder-scheduler",
        messages=0,
        consumers=3,
        auto_delete=False)
    assert queues.data[3] == RabbitMQQueues.QueueInfo(
        name="cinder-scheduler_fanout_9aed9fbc3d4249289f2cb5ea04c062ab",
        messages=8145,
        consumers=0,
        auto_delete=True)
    assert queues.data[3].name == "cinder-scheduler_fanout_9aed9fbc3d4249289f2cb5ea04c062ab"

    many_queues = RabbitMQQueues(context_wrap(MORE_QUEUES))
    assert many_queues.data[1].name == "dhcp_agent.undercloud-per720xd.default.redhat.local"
    assert many_queues.data[1].auto_delete is False

    with pytest.raises(ParseException):
        queues = RabbitMQQueues(context_wrap(QUEUES_BAD_1))

    with pytest.raises(ValueError):
        queues = RabbitMQQueues(context_wrap(QUEUES_BAD_2))

    with pytest.raises(ParseException):
        queues = RabbitMQQueues(context_wrap(QUEUES_BAD_3))



"""
DesignateConf - file ``/etc/designate/designate.conf``
======================================================
"""
from insights.core import IniConfigFile
from insights.core.filters import add_filter
from insights.core.plugins import parser
from insights.specs import Specs

ADDITIONAL_FILTERS = [
    "[",
    "state_path",
    "root_helper",
    "debug",
    "log_dir",
    "www_authenticate_uri",
    "project_name",
    "project_domain_name",
    "driver"
]
add_filter(Specs.designate_conf, ADDITIONAL_FILTERS)


@parser(Specs.designate_conf)
class DesignateConf(IniConfigFile):
    """
    This class provides parsing for the files:
        ``/var/lib/config-data/puppet-generated/designate/etc/designate/designate.conf``
        ``/etc/designate/designate.conf``

    Sample input data is in the format::

        [DEFAULT]
        state_path=/var/lib/designate
        root_helper=sudo designate-rootwrap /etc/designate/rootwrap.conf
        debug=True
        log_dir=/var/log/designate

        [keystone_authtoken]
        www_authenticate_uri=http://localhost:5000
        project_name=service
        project_domain_name=Default

        [oslo_messaging_notifications]
        driver=messagingv2

    Examples:
        >>> type(conf)
        <class 'insights.parsers.designate_conf.DesignateConf'>
        >>> conf.sections()
        ['keystone_authtoken', 'oslo_messaging_notifications']
        >>> conf.has_option('DEFAULT', 'debug')
        True
        >>> conf.get("oslo_messaging_notifications", "driver")
        'messagingv2'
    """
    pass



"""
FapolicydRules - file ``/etc/fapolicyd/rules.d/*.rules``
========================================================
"""

from insights import parser
from insights.core import LogFileOutput
from insights.specs import Specs


@parser(Specs.fapolicyd_rules)
class FapolicydRules(LogFileOutput):
    """
    Parse the content of ``/etc/fapolicyd/rules.d/*.rules`` file.

    .. note::

        The rules do not require to get the parsed result currently.
        It just need to check if it contains specific lines, so use
        :class:`insights.core.LogFileOutput` as the base class.

    Sample input::

        deny_audit perm=any pattern=ld_so : all
        deny_audit perm=any pattern=ld_preload : all

    Examples:
        >>> from insights.parsers.fapolicyd_rules import FapolicydRules
        >>> FapolicydRules.last_scan('ld_so_deny_audit_test', 'deny_audit perm=any pattern=ld_so : all')
        >>> type(fapolicyd_rules)
        <class 'insights.parsers.fapolicyd_rules.FapolicydRules'>
        >>> fapolicyd_rules.ld_so_deny_audit_test.get('raw_message')
        'deny_audit perm=any pattern=ld_so : all'
    """
    pass



import doctest
import pytest

from insights.core.exceptions import ContentException, ParseException, SkipComponent
from insights.parsers import du
from insights.parsers.du import DiskUsage
from insights.tests import context_wrap

# du -s /var/lib/pgsql
DU_VAR_LIB = """
186724	/var/lib/pgsql
""".strip()

# du -s /var/lib/*
DU_VAR_LIB_STAR = """
56	/var/lib/alternatives
4	/var/lib/logrotate
5492	/var/lib/mlocate
20	/var/lib/NetworkManager
186484	/var/lib/pgsql
856	/var/lib/rhsm
110712	/var/lib/rpm
4	/var/lib/rsyslog
64	/var/lib/systemd
15200	/var/lib/yum
""".strip()

# du -sh / containing only error lines
DU_ACCESS_ERROR = """
/bin/du: cannot access '/proc/17405/task/17405/fd/4': No such file or directory
/bin/du: cannot access '/proc/17405/task/17405/fdinfo/4': No such file or directory
""".strip()

# du -s / containing error lines and a valid value
DU_ACCESS_ERROR_OKAY = """
/bin/du: cannot access '/proc/17405/task/17405/fd/4': No such file or directory
5904560	/
""".strip()

# Valid output with spaces in path
DU_SPACES = """
102400	/mnt/abc xyz
""".strip()

# du -a inside a directory
DU_RELATIVE = """
1652	./messages
1652	.
"""

# du --blocks=4K , 4k blocks but no suffix unit
DU_4K_VAR_LOG = """
3	/var/log/tuned
3969	/var/log/rhsm
85	/var/log/httpd
3	/var/log/squid
""".strip()

# du -sh / containing only one error line
# CommandParser will throw ContentException as
# this contains single line with "No such file or directory"
DU_ACCESS_ERROR_SINGLE = """
/bin/du: cannot access '/proc/17405/task/17405/fd/4': No such file or directory
""".strip()

# 1M Blocks, Parser only supports integer values without unit suffix
DU_M_VAR_LOG = """
1M	/var/log/tuned
144M	/var/log/rhsm
26M	/var/log/httpd
5M	/var/log/squid
""".strip()

# du -h /var/log throwing exception
DU_H = """
12K	/var/log/tuned
144M	/var/log/rhsm
26M	/var/log/httpd
4.2M	/var/log/squid
""".strip()

DU_INVALID_1 = """
20	/var/lib/NetworkManager
110712
""".strip()

DU_INVALID_2 = """
2A	/var/lib/NetworkManager
""".strip()


def test_du():
    du = DiskUsage(context_wrap(DU_VAR_LIB))
    assert len(du) == 1
    assert '/var/lib/pgsql' in du
    assert du.get('/var/lib/pgsql') == 186724
    assert '/var/some/fake' not in du
    assert du.get('/Fake') is None
    assert du == {'/var/lib/pgsql': 186724}

    du = DiskUsage(context_wrap(DU_VAR_LIB_STAR))
    assert len(du) == 10
    assert '/var/lib/rpm' in du
    assert du.get('/var/lib/rpm') == 110712
    assert du == {'/var/lib/alternatives': 56, '/var/lib/logrotate': 4, '/var/lib/mlocate': 5492, '/var/lib/NetworkManager': 20, '/var/lib/pgsql': 186484, '/var/lib/rhsm': 856, '/var/lib/rpm': 110712, '/var/lib/rsyslog': 4, '/var/lib/systemd': 64, '/var/lib/yum': 15200}

    du = DiskUsage(context_wrap(DU_ACCESS_ERROR_OKAY))
    assert len(du) == 1
    assert '/' in du
    assert du.get('/') == 5904560

    du = DiskUsage(context_wrap(DU_SPACES))
    assert du.get('/mnt/abc xyz') == 102400
    assert du == {'/mnt/abc xyz': 102400}

    du = DiskUsage(context_wrap(DU_4K_VAR_LOG))
    assert len(du) == 4
    assert '/var/log/httpd' in du
    assert du.get('/var/log/httpd') == 85


def test_du_bad():

    with pytest.raises(SkipComponent) as exc:
        DiskUsage(context_wrap(DU_ACCESS_ERROR))
    assert 'No data parsed' in str(exc)

    with pytest.raises(ParseException) as exc:
        DiskUsage(context_wrap(DU_RELATIVE))
    assert 'Relative paths not supported' in str(exc)

    with pytest.raises(ContentException) as exc:
        DiskUsage(context_wrap(DU_ACCESS_ERROR_SINGLE))
    assert 'No such file or directory' in str(exc)

    with pytest.raises(ParseException) as exc:
        DiskUsage(context_wrap(DU_M_VAR_LOG))
    assert 'Could not parse line' in str(exc)

    with pytest.raises(ParseException) as exc:
        DiskUsage(context_wrap(DU_H))
    assert 'Could not parse line' in str(exc)

    with pytest.raises(ParseException) as exc:
        DiskUsage(context_wrap(DU_INVALID_1))
    assert 'Could not parse line' in str(exc)

    with pytest.raises(ParseException) as exc:
        DiskUsage(context_wrap(DU_INVALID_2))
    assert 'Could not parse line' in str(exc)


def test_du_doc_examples():
    env = {'disk_usage': DiskUsage(context_wrap(DU_VAR_LIB_STAR))}
    failed, total = doctest.testmod(du, globs=env)
    assert failed == 0



"""
CpupowerFrequencyInfo - Commands ``cpupower -c all frequency-info``
===================================================================
"""
from insights.core import CommandParser
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.cpupower_frequency_info)
class CpupowerFrequencyInfo(CommandParser, dict):
    """
    Class for parsing the output of `cpupower -c all frequency-info` command.

    Typical output of the command is::

        analyzing CPU 0:
          driver: intel_pstate
          CPUs which run at the same hardware frequency: 0
          CPUs which need to have their frequency coordinated by software: 0
          maximum transition latency:  Cannot determine or is not supported.
          hardware limits: 800 MHz - 3.00 GHz
          available cpufreq governors: performance powersave
          current policy: frequency should be within 800 MHz and 3.00 GHz.
                          The governor "performance" may decide which speed to use
                          within this range.
          current CPU frequency: Unable to call hardware
          current CPU frequency: 1.22 GHz (asserted by call to kernel)
          boost state support:
            Supported: yes
            Active: yes
            2700 MHz max turbo 4 active cores
            2800 MHz max turbo 3 active cores
            2900 MHz max turbo 2 active cores
            3000 MHz max turbo 1 active cores
        analyzing CPU 1:
          driver: intel_pstate
          CPUs which run at the same hardware frequency: 1
          CPUs which need to have their frequency coordinated by software: 1
          maximum transition latency:  Cannot determine or is not supported.
          hardware limits: 800 MHz - 3.00 GHz
          available cpufreq governors: performance powersave
          current policy: frequency should be within 800 MHz and 3.00 GHz.
                          The governor "performance" may decide which speed to use
                          within this range.
          current CPU frequency: Unable to call hardware
          current CPU frequency: 871 MHz (asserted by call to kernel)
          boost state support:
            Supported: yes
            Active: yes

    Raises:
        SkipComponent: When input is empty.
        ParseException: When input cannot be parsed.

    Examples:

        >>> type(cpupower_frequency_info)
        <class 'insights.parsers.cpupower_frequency_info.CpupowerFrequencyInfo'>
        >>> cpupower_frequency_info['analyzing CPU 0']['driver']
        'intel_pstate'
        >>> cpupower_frequency_info['analyzing CPU 0']['boost state support']['Supported']
        'yes'
        >>> cpupower_frequency_info['analyzing CPU 0']['boost state support']['Active']
        'yes'
        >>> cpupower_frequency_info['analyzing CPU 1']['current policy']
        'frequency should be within 800 MHz and 3.00 GHz. The governor "performance" may decide which speed to use within this range.'
        >>> cpupower_frequency_info['analyzing CPU 0']['boost state support']['2700 MHz max turbo 4 active cores']
        True
    """

    def parse_content(self, content):
        if not content:
            raise SkipComponent("Empty content")
        if len(content) < 10 or not ('analyzing CPU' in content[0]):
            raise ParseException("Incorrect content: '{0}'".format(content))

        power_frequency_info = {}
        current_data = power_frequency_info
        parents = []
        prev_shift = 0
        prev_key = None

        for line in content:
            shift = (len(line) - len(line.lstrip()))
            if shift > prev_shift and (shift - prev_shift) > 2:
                current_data[prev_key] = ' '.join([current_data[prev_key], line.strip()])
            else:
                if shift > prev_shift:
                    current_data[prev_key] = {}
                    parents.append(current_data)
                    current_data = current_data[prev_key]
                elif shift < prev_shift:
                    while prev_shift - shift >= 2:
                        current_data = parents.pop()
                        prev_shift = prev_shift - 2
                key, value = line.strip(), True
                if ":" in line:
                    key, value = [l.strip() for l in line.strip().split(":", 1)]
                current_data[key] = value
                prev_shift = shift
                prev_key = key
        self.update(power_frequency_info)



"""
Parted Parsers
==============

Classes to parse ``parted`` command information.

Parsers provided by this module include:

PartedL - command ``parted -l -s``
----------------------------------

"""
from insights.core import CommandParser
from insights.core.exceptions import ParseException
from insights.core.plugins import parser
from insights.parsers import parse_fixed_table
from insights.specs import Specs


class Partition(object):
    """Class to contain information for one partition.

    Represents the values from one row of the partition information from the
    ``parted`` command.  Column names have been converted to lowercase and are
    provided as attributes.  Column names may vary so the ``get`` method may
    be used to check for the presence of a column.

    Attributes:
        data (dict): Dictionary of partition information keyed by column names
            in lowercase.
    """
    def __init__(self, data):
        self.data = data

    @property
    def number(self):
        """str: Partition number."""
        return self.data.get('number')

    @property
    def start(self):
        """str: Starting location for the partition."""
        return self.data.get('start')

    @property
    def end(self):
        """str: Ending location for the partition."""
        return self.data.get('end')

    @property
    def size(self):
        """str: Size of the partition."""
        return self.data.get('size')

    @property
    def file_system(self):
        """str: File system type."""
        return self.data.get('file_system')

    @property
    def type(self):
        """str: File system type."""
        return self.data.get('type')

    @property
    def flags(self):
        """str: Partition flags."""
        return self.data.get('flags')

    def get(self, item):
        """Get information for column ``item`` or ``None`` if not present."""
        return self.data.get(item)


class PartedDevice(object):
    """Class to contain information for one device of ``parted`` command output.

    The columns of partation table may vary depending upon the type of device.

    Attributes:
        device_info (dict): Dictionary of information of this device.
        partitions (list): The partitions of this device, as Partition objects.
        boot_partition (Partition): the first partition marked as bootable,
            or ``None`` if one was not found.

    Raises:
        ParseException: Raised if command output of this device indicates
            "error" or "warning" in first line, or if "disk" field is not
            present, or if there is an error parsing the data.
    """

    @property
    def data(self):
        """dict: Device information."""
        return self.device_info

    @property
    def disk(self):
        """str: Disk information."""
        return self.device_info['disk']

    @property
    def logical_sector_size(self):
        """str: Logical part of sector size."""
        if self._sector_size:
            return self._sector_size[0]

    @property
    def physical_sector_size(self):
        """str: Physical part of sector size."""
        if self._sector_size:
            return self._sector_size[1]

    def get(self, item):
        """Returns a value for the specified ``item`` key."""
        return self.device_info.get(item)

    def __init__(self, content):
        # If device was not present output is error message
        if content[0].startswith("Error") or content[0].startswith("Warning"):
            raise ParseException("PartedL content indicates an error %s" % content[0])

        dev_info = {}
        table_lines = []
        for line in content:
            if not line.strip():
                continue
            if ':' in line:
                label_value = line.split(':')
                label = label_value[0].strip().lower()
                if len(label_value) == 2:
                    value = label_value[1].strip()
                    value = value if value else None
                    # Single word labels
                    if ' ' not in label:
                        dev_info[label] = value
                    else:
                        if label.startswith("disk") and '/' in label:
                            disk_parts = label.split()
                            dev_info['disk'] = disk_parts[1].strip()
                            dev_info['size'] = value
                        elif label.startswith("sector"):
                            dev_info['sector_size'] = value
                        else:
                            label = label.replace(' ', '_')
                            dev_info[label] = value
            else:
                table_lines.append(line)

        if 'disk' not in dev_info:
            raise ParseException("PartedL unable to locate Disk in content")

        # Now construct the partition table from the fixed table
        partitions = []
        if table_lines:
            table_lines[0] = table_lines[0].replace('File system', 'File_system').lower()
            partitions = parse_fixed_table(table_lines)

        self.partitions = [Partition(n) for n in partitions]
        self.boot_partition = None
        self._sector_size = None
        # If we got any partitions, find the first boot partition
        for part in partitions:
            if 'flags' in part and 'boot' in part['flags']:
                self.boot_partition = Partition(part)
                break
        self.device_info = dev_info
        if 'sector_size' in self.device_info:
            self._sector_size = self.device_info['sector_size'].split('/', 1)
            if len(self._sector_size) != 2:
                self._sector_size = None


@parser(Specs.parted__l)
class PartedL(CommandParser):
    """Class to represent attributes of the ``parted -l -s`` command output.

    Attributes:
        devices_info (list): The devices found in the output, as PartedDevice
            objects.

    Typical content of the ``parted -l -s`` command output looks like::

        Model: ATA TOSHIBA MG04ACA4 (scsi)
        Disk /dev/sda: 4001GB
        Sector size (logical/physical): 512B/512B
        Partition Table: gpt
        Disk Flags: pmbr_boot

        Number  Start   End     Size    File system  Name  Flags
         1      1049kB  2097kB  1049kB                     bios_grub
         2      2097kB  526MB   524MB   xfs
         3      526MB   4001GB  4000GB                     lvm


        Model: IBM 2107900 (scsi)
        Disk /dev/sdb: 2147MB
        Sector size (logical/physical): 512B/512B
        Partition Table: msdos

        Number  Start   End     Size    Type     File system  Flags
         1      32.3kB  2580kB  2548kB  primary

    The columns of partation table may vary depending upon the type of device.

    Note:
        The examples in this module may be executed with the following command:

        ``python -m insights.parsers.parted``

    Examples:
        >>> [device.disk for device in parted_l_results]
        ['/dev/sda', '/dev/sdb']
        >>> parted_info = parted_l_results.get('/dev/sda')
        >>> sorted(parted_info.device_info.items())
        [('disk', '/dev/sda'), ('disk_flags', 'pmbr_boot'), ('model', 'ATA TOSHIBA MG04ACA4 (scsi)'), ('partition_table', 'gpt'), ('sector_size', '512B/512B'), ('size', '4001GB')]
        >>> parted_info.device_info['model']
        'ATA TOSHIBA MG04ACA4 (scsi)'
        >>> parted_info.disk
        '/dev/sda'
        >>> parted_info.logical_sector_size
        '512B'
        >>> parted_info.physical_sector_size
        '512B'
        >>> parted_info.boot_partition
        >>> parted_info.device_info['disk_flags']
        'pmbr_boot'
        >>> len(parted_info.partitions)
        3
        >>> sorted(parted_info.partitions[0].data.items())
        [('end', '2097kB'), ('file_system', ''), ('flags', 'bios_grub'), ('name', ''), ('number', '1'), ('size', '1049kB'), ('start', '1049kB')]
        >>> parted_info.partitions[0].number
        '1'
        >>> parted_info.partitions[0].start
        '1049kB'
        >>> parted_info.partitions[0].end
        '2097kB'
        >>> parted_info.partitions[0].size
        '1049kB'
        >>> parted_info.partitions[0].file_system
        ''
        >>> parted_info.partitions[0].type
        >>> parted_info.partitions[0].flags
        'bios_grub'

    """

    def get(self, disk_name):
        """Returns a value for the specified ``item`` key."""
        for dev in self.devices_info:
            if dev.disk == disk_name:
                return dev

    def __iter__(self):
        for dev in self.devices_info:
            yield dev

    def parse_content(self, content):
        """Divide "content" into each device section by two empty lines."""

        device_tables = []

        # Divide "content" into each Device Section by two empty lines.
        idx, this_start_idx = 0, 0
        is_last_line_empty = False
        for idx, raw_line in enumerate(content):
            if len(raw_line.strip()) == 0:
                if is_last_line_empty:
                    # Fount this line as the second blank line
                    device_tables.append(content[this_start_idx:idx - 1])
                    this_start_idx = idx + 1
                    is_last_line_empty = False
                else:
                    is_last_line_empty = True
            else:
                is_last_line_empty = False

        # Take left "content" as the last Device Section, skip the empty case
        if this_start_idx < len(content) - 1:
            device_tables.append(content[this_start_idx:])

        devices_info = []
        for dev_table in device_tables:
            try:
                this_parsed_dev_table = PartedDevice(dev_table)
                devices_info.append(this_parsed_dev_table)
            except ParseException:
                # Mute any exception from PartedDevice parsing.
                # Keep only fully parsed Device into the final devices_info.
                pass

        self.devices_info = devices_info



import pytest

from mock.mock import patch

from insights.combiners.ps import Ps
from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.parsers.mount import ProcMounts
from insights.specs.datasources.httpd import httpd_cmds, httpd_on_nfs
from insights.tests import context_wrap


MOUNT_DATA = """
/dev/mapper/root / ext4 rw,relatime,barrier=1,data=ordered 0 0
/dev/mapper/httpd1 /httpd1 nfs4 rw,relatime,vers=4,barrier=1,data=ordered 0 0
/dev/mapper/httpd2 /httpd2 nfs4 rw,relatime,vers=4,barrier=1,data=ordered 0 0
""".strip()
NFS_LSOF_666 = """
zsh     3520 httpd    3r   REG  253,0   6940392    648646 /httpd1
zsh     3520 httpd   11r   REG  253,0   9253600    648644 /httpd1
zsh     3520 httpd   12u   CHR    5,0       0t0      1034 /dev/tty
""".strip()
NFS_LSOF_777 = """
zsh     3520 httpd    3r   REG  253,0   6940392    648646 /httpd2
zsh     3520 httpd   11r   REG  253,0   9253600    648644 /httpd2
zsh     3520 httpd   12u   CHR    5,0       0t0      1034 /httpd2
""".strip()


class FakeContext(HostContext):
    def shell_out(self, cmd, split=True, timeout=None, keep_rc=False, env=None, signum=None):
        if 'pgrep' in cmd:
            return '666', '777'
        elif 'lsof -p 666' in cmd:
            return NFS_LSOF_666.splitlines()
        elif 'lsof -p 777' in cmd:
            return NFS_LSOF_777.splitlines()

        raise Exception


# The ``get_running_commands()`` is tested in:
# - insights/tests/datasources/test_get_running_commands.py
# Here, we do not test it
@patch('insights.specs.datasources.httpd.get_running_commands')
def test_httpd_cmds(run_cmds):
    broker = {Ps: None, HostContext: None}
    httpds = ['/usr/sbin/httpd/', '/opt/rh/httpd24/root/usr/sbin/httpd']
    run_cmds.return_value = httpds
    result = httpd_cmds(broker)
    assert result == httpds

    run_cmds.return_value = []
    with pytest.raises(SkipComponent):
        httpd_cmds(broker)


@patch('insights.specs.datasources.httpd.get_running_commands')
def test_httpd_on_nfs(run_cmds):
    broker = {ProcMounts: ProcMounts(context_wrap(MOUNT_DATA)), HostContext: FakeContext()}
    result = httpd_on_nfs(broker)
    assert '"http_ids": ["666", "777"]' in result.content[0]
    assert '"open_nfs_files": 5' in result.content[0]
    assert '"nfs_mounts": ["/httpd1", "/httpd2"]' in result.content[0]



import pytest

from insights import SkipComponent
from insights.combiners.virt_what import VirtWhat
from insights.components.virtualization import IsBareMetal
from insights.parsers.virt_what import VirtWhat as VWP
from insights.tests import context_wrap


def test_is_bare_metal():
    virt_what = VirtWhat(None, VWP(context_wrap("")))
    result = IsBareMetal(virt_what)
    assert isinstance(result, IsBareMetal)

    virt_what = VirtWhat(None, VWP(context_wrap("kvm")))
    with pytest.raises(SkipComponent):
        IsBareMetal(virt_what)



"""
IpaupgradeLog - file ``/var/log/ipaupgrade.log``
================================================

This file records the information of IPA server upgrade process while
executing command ``ipa-server-upgrade``
"""

from .. import LogFileOutput, parser
from insights.specs import Specs


@parser(Specs.ipaupgrade_log)
class IpaupgradeLog(LogFileOutput):
    """
    This parser is used to parse the content of file `/var/log/ipaupgrade.log`.

    .. note::
        Please refer to its super-class :class:`insights.core.LogFileOutput`

    Typical content of ``ipaupgrade.log`` file is::

        2017-08-07T07:36:50Z DEBUG Starting external process
        2017-08-07T07:36:50Z DEBUG args=/bin/systemctl is-active pki-tomcatd@pki-tomcat.service
        2017-08-07T07:36:50Z DEBUG Process finished, return code=0
        2017-08-07T07:36:50Z DEBUG stdout=active
        2017-08-07T07:41:50Z ERROR IPA server upgrade failed: Inspect /var/log/ipaupgrade.log and run command ipa-server-upgrade manually.

    Example:
        >>> ipaupgradelog = shared[IpaupgradeLog]
        >>> len(list(log.get('DEBUG')))
        4
        >>> from datetime import datetime
        >>> len(log.get_after(datetime(2017, 8, 7, 7, 37, 30)))
        1
    """
    time_format = '%Y-%m-%dT%H:%M:%SZ'



from insights.tests import context_wrap
from insights.parsers import sysconfig
from insights.parsers.sysconfig import ChronydSysconfig, DockerSysconfig, DockerSysconfigStorage
from insights.parsers.sysconfig import HttpdSysconfig, IrqbalanceSysconfig
from insights.parsers.sysconfig import LibvirtGuestsSysconfig, MemcachedSysconfig
from insights.parsers.sysconfig import MongodSysconfig, NtpdSysconfig
from insights.parsers.sysconfig import PrelinkSysconfig, VirtWhoSysconfig
from insights.parsers.sysconfig import SshdSysconfig
from insights.parsers.sysconfig import Up2DateSysconfig, PuppetserverSysconfig
from insights.parsers.sysconfig import NetconsoleSysconfig, ForemanTasksSysconfig
from insights.parsers.sysconfig import DockerStorageSetupSysconfig, DirsrvSysconfig
from insights.parsers.sysconfig import CorosyncSysconfig
from insights.parsers.sysconfig import IfCFGStaticRoute
from insights.parsers.sysconfig import NetworkSysconfig
from insights.parsers.sysconfig import GrubSysconfig
from insights.parsers.sysconfig import OracleasmSysconfig
from insights.parsers.sysconfig import NfsSysconfig
from insights.parsers.sysconfig import StonithSysconfig
import doctest


CHRONYDSYSCONFIG = """
OPTIONS="-d"
#HIDE="me"
""".strip()

NTPDSYSCONFIG = """
OPTIONS="-x -g"
#HIDE="me"
""".strip()

DOCKERSYSCONFIG = """
OPTIONS="--selinux-enabled"
DOCKER_CERT_PATH="/etc/docker"
""".strip()

HTTPDSYSCONFIG = """
HTTPD=/usr/sbin/httpd.worker
OPTIONS=
""".strip()

IRQBALANCESYSCONFIG = """
#IRQBALANCE_ONESHOT=yes
IRQBALANCE_BANNED_CPUS=f8
IRQBALANCE_ARGS="-d"
""".strip()

VIRTWHOSYSCONFIG = """
# Register ESX machines using vCenter
# VIRTWHO_ESX=0
# Register guests using RHEV-M
VIRTWHO_RHEVM=1
# Options for RHEV-M mode
VIRTWHO_RHEVM_OWNER=
TEST_OPT="A TEST"
""".strip()

MONGODSYSCONFIG = """
OPTIONS="--quiet -f /etc/mongod.conf"
""".strip()

MEMCACHEDSYSCONFIG = """
PORT="11211"
USER="memcached"
# max connection 2048
MAXCONN="2048"
CACHESIZE="4096"
OPTIONS="-U 0 -l 127.0.0.1"
""".strip()

LIBVIRTGUESTSSYSCONFIG = """
ON_BOOT=ignore
""".strip()

PRELINKSYSCONFIG = """
PRELINKING=no
PRELINK_OPTS=-mR
""".strip()

UP2DATESYSCONFIG = """
serverURL[comment]=Remote server URL
#serverURL=https://rhnproxy.glb.tech.markit.partners
serverURL=https://rhnproxy.glb.tech.markit.partners/XMLRPC
""".strip()

SSHDSYSCONFIG = """
# Configuration file for the sshd service.

# The server keys are automatically generated if they are missing.
# To change the automatic creation, adjust sshd.service options for
# example using  systemctl enable sshd-keygen@dsa.service  to allow creation
# of DSA key or  systemctl mask sshd-keygen@rsa.service  to disable RSA key
# creation.

# System-wide crypto policy:
# To opt-out, uncomment the following line
# CRYPTO_POLICY=
CRYPTO_POLICY=
""".strip()

PUPPETSERVERCONFIG = """
USER="puppet"
GROUP="puppet"
INSTALL_DIR="/opt/puppetlabs/server/apps/puppetserver"
CONFIG="/etc/puppetlabs/puppetserver/conf.d"
START_TIMEOUT=300
""".strip()

NETCONSOLESYSCONFIG = """
LOCALPORT=6666
""".strip()

FOREMANTASKSYSCONFG = """
FOREMAN_USER=foreman
BUNDLER_EXT_HOME=/usr/share/foreman
RAILS_ENV=production
FOREMAN_LOGGING=warn
""".strip()

DOCKERSTORAGESETUPSYSCONFG = """
VG=vgtest
AUTO_EXTEND_POOL=yes
##name = mydomain
POOL_AUTOEXTEND_THRESHOLD=60
POOL_AUTOEXTEND_PERCENT=20
""".strip()

DIRSRVSYSCONFG = """
#STARTPID_TIME=10 ; export STARTPID_TIME
#PID_TIME=600 ; export PID_TIME
KRB5CCNAME=/tmp/krb5cc_995
KRB5_KTNAME=/etc/dirsrv/ds.keytab
""".strip()

COROSYNCSYSCONFIG = """
# COROSYNC_INIT_TIMEOUT specifies number of seconds to wait for corosync
# initialization (default is one minute).
COROSYNC_INIT_TIMEOUT=60
# COROSYNC_OPTIONS specifies options passed to corosync command
# (default is no options).
# See "man corosync" for detailed descriptions of the options.
COROSYNC_OPTIONS=""
""".strip()
CONTEXT_PATH_DEVICE_1 = "etc/sysconfig/network-scripts/route-test-net"

STATIC_ROUTE_1 = """
ADDRESS0=10.65.223.0
NETMASK0=255.255.254.0
GATEWAY0=10.65.223.1
""".strip()

DOCKER_CONFIG_STORAGE = """
DOCKER_STORAGE_OPTIONS="--storage-driver devicemapper --storage-opt dm.fs=xfs --storage-opt dm.thinpooldev=/dev/mapper/dockervg-docker--pool --storage-opt dm.use_deferred_removal=true --storage-opt dm.use_deferred_deletion=true"
""".strip()

NETWORK_SYSCONFIG = """
NETWORKING=yes
HOSTNAME=rhel7-box
GATEWAY=172.31.0.1
NM_BOND_VLAN_ENABLED=no
""".strip()

GRUB_SYSCONFIG = """

GRUB_TIMEOUT=1
GRUB_DISTRIBUTOR="$(sed 's, release .*$,,g' /etc/system-release)"
GRUB_DEFAULT=saved
GRUB_DISABLE_SUBMENU=true
GRUB_TERMINAL_OUTPUT="console"
GRUB_CMDLINE_LINUX="console=ttyS0 console=ttyS0,115200n8 no_timer_check net.ifnames=0 crashkernel=auto"
GRUB_DISABLE_RECOVERY="true"
GRUB_ENABLE_BLSCFG=true
""".strip()

ORACLEASM_SYSCONFIG = """
#
# This is a configuration file for automatic loading of the Oracle
# Automatic Storage Management library kernel driver.  It is generated
# By running /etc/init.d/oracleasm configure.  Please use that method
# to modify this file
#
# ORACLEASM_ENABELED: 'true' means to load the driver on boot.
ORACLEASM_ENABLED=true
# ORACLEASM_UID: Default user owning the /dev/oracleasm mount point.
ORACLEASM_UID=oracle
# ORACLEASM_GID: Default group owning the /dev/oracleasm mount point.
ORACLEASM_GID=oinstall
# ORACLEASM_SCANBOOT: 'true' means scan for ASM disks on boot.
ORACLEASM_SCANBOOT=true
# ORACLEASM_SCANORDER: Matching patterns to order disk scanning
ORACLEASM_SCANORDER="dm"
# ORACLEASM_SCANEXCLUDE: Matching patterns to exclude disks from scan
ORACLEASM_SCANEXCLUDE="sd"
""".strip()

NFS_SYSCONFIG = """
# Optional arguments passed to rpc.nfsd. See rpc.nfsd(8)
RPCNFSDARGS="--rdma=20049"
# Port rpc.statd should listen on.
#STATD_PORT=662
# Enable usage of gssproxy. See gssproxy-mech(8).
GSS_USE_PROXY="yes"
# Optional arguments passed to rpc.svcgssd. See rpc.svcgssd(8)
RPCSVCGSSDARGS=""
# Optional arguments passed to blkmapd. See blkmapd(8)
BLKMAPDARGS=""
RPCNFSDCOUNT=256
""".strip()

STONITH_CONFIG = """
retry=3
retry-sleep=2
verbose=yes    # optional
""".strip()


def test_sysconfig_doc():
    env = {
            'chronyd_syscfg': ChronydSysconfig(context_wrap(CHRONYDSYSCONFIG)),
            'ntpd_syscfg': NtpdSysconfig(context_wrap(NTPDSYSCONFIG)),
            'docker_syscfg': DockerSysconfig(context_wrap(DOCKERSYSCONFIG)),
            'docker_syscfg_storage': DockerSysconfigStorage(context_wrap(DOCKER_CONFIG_STORAGE)),
            'httpd_syscfg': HttpdSysconfig(context_wrap(HTTPDSYSCONFIG)),
            'irqb_syscfg': IrqbalanceSysconfig(context_wrap(IRQBALANCESYSCONFIG)),
            'vwho_syscfg': VirtWhoSysconfig(context_wrap(VIRTWHOSYSCONFIG)),
            'mongod_syscfg': MongodSysconfig(context_wrap(MONGODSYSCONFIG)),
            'memcached_syscfg': MemcachedSysconfig(context_wrap(MEMCACHEDSYSCONFIG)),
            'libvirt_guests_syscfg': LibvirtGuestsSysconfig(context_wrap(LIBVIRTGUESTSSYSCONFIG)),
            'prelink_syscfg': PrelinkSysconfig(context_wrap(PRELINKSYSCONFIG)),
            'u2d_syscfg': Up2DateSysconfig(context_wrap(UP2DATESYSCONFIG)),
            'netcs_syscfg': NetconsoleSysconfig(context_wrap(NETCONSOLESYSCONFIG)),
            'sshd_syscfg': SshdSysconfig(context_wrap(SSHDSYSCONFIG)),
            'pps_syscfg': PuppetserverSysconfig(context_wrap(PUPPETSERVERCONFIG)),
            'ft_syscfg': ForemanTasksSysconfig(context_wrap(FOREMANTASKSYSCONFG)),
            'dss_syscfg': DockerStorageSetupSysconfig(context_wrap(DOCKERSTORAGESETUPSYSCONFG)),
            'dirsrv_syscfg': DirsrvSysconfig(context_wrap(DIRSRVSYSCONFG)),
            'cs_syscfg': CorosyncSysconfig(context_wrap(COROSYNCSYSCONFIG)),
            'conn_info': IfCFGStaticRoute(context_wrap(STATIC_ROUTE_1, CONTEXT_PATH_DEVICE_1)),
            'net_syscfg': NetworkSysconfig(context_wrap(NETWORK_SYSCONFIG)),
            'nfs_syscfg': NfsSysconfig(context_wrap(NFS_SYSCONFIG)),
            'grub_syscfg': GrubSysconfig(context_wrap(GRUB_SYSCONFIG)),
            'oracleasm_syscfg': OracleasmSysconfig(context_wrap(ORACLEASM_SYSCONFIG)),
            'stonith_syscfg': StonithSysconfig(context_wrap(STONITH_CONFIG))
          }
    failed, total = doctest.testmod(sysconfig, globs=env)
    assert failed == 0



"""
Custom datasources to get a list of modules to check the detailed module info.
"""
from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.filters import get_filters
from insights.core.plugins import datasource
from insights.parsers.lsmod import LsMod
from insights.specs import Specs


@datasource(LsMod, HostContext)
def kernel_module_filters(broker):
    """
    Return a string of a list of modules from the spec filter,
    separated with space.
    """
    filters = sorted((get_filters(Specs.modinfo_modules)))
    if filters:
        loaded_modules = []
        for item in filters:
            module_list = [module for module in broker[LsMod].data if item in module]
            if module_list:
                loaded_modules.extend(module_list)
        if loaded_modules:
            return ' '.join(loaded_modules)
        raise SkipComponent
    raise SkipComponent



"""
Custom datasources for ``ls`` commands
"""
from insights.core.context import HostContext
from insights.core.dr import get_name
from insights.core.exceptions import SkipComponent
from insights.core.filters import get_filters
from insights.core.plugins import datasource
from insights.specs import Specs


def _list_items(spec):
    """Return a tuple of directories according to the spec filters."""
    filters = sorted(get_filters(spec))
    if filters:
        if len(filters) == 1 and 'R' not in get_name(spec).split('_')[-2]:
            """
            .. note::
                Insert a non-existing directory when there is only ONE target for
                the datasource to list, to make sure the directory be outputted.

                ==============================================================
                # list a single dir '/mnt'
                $ ls -lan /mnt
                total 0
                drwxr-xr-x.  2 0 0   6 Jun 21  2021 .
                dr-xr-xr-x. 17 0 0 224 Apr 17 16:45 ..
                --------------------------------------------------------------
                # list a single dir with this patch
                $ ls -lan /mnt _non_existing_
                ls: cannot access ' _non_existing_': No such file or directory
                /mnt:                      # <--<< the dir is outputted here
                total 0
                drwxr-xr-x.  2 0 0   6 Jun 21  2021 .
                dr-xr-xr-x. 17 0 0 224 Apr 17 16:45 ..
                ==============================================================
            """
            filters.append('_non_existing_')
        return ' '.join(filters)
    raise SkipComponent


@datasource(HostContext)
def list_with_la(broker):
    return _list_items(Specs.ls_la_dirs)


@datasource(HostContext)
def list_with_la_filtered(broker):
    return _list_items(Specs.ls_la_filtered_dirs)


@datasource(HostContext)
def list_with_lan(broker):
    return _list_items(Specs.ls_lan_dirs)


@datasource(HostContext)
def list_with_lan_filtered(broker):
    return _list_items(Specs.ls_lan_filtered_dirs)


@datasource(HostContext)
def list_with_lanL(broker):
    return _list_items(Specs.ls_lanL_dirs)


@datasource(HostContext)
def list_with_lanR(broker):
    return _list_items(Specs.ls_lanR_dirs)


@datasource(HostContext)
def list_with_lanRL(broker):
    return _list_items(Specs.ls_lanRL_dirs)


@datasource(HostContext)
def list_with_lanRZ(broker):
    return _list_items(Specs.ls_lanRZ_dirs)


@datasource(HostContext)
def list_with_lanZ(broker):
    return _list_items(Specs.ls_lanZ_dirs)



import pytest

from mock.mock import Mock

from insights.core.exceptions import SkipComponent
from insights.parsers.nmcli import NmcliConnShow
from insights.specs.datasources.ethernet import LocalSpecs, interfaces, team_interfaces
from insights.tests import context_wrap


IP_LINK = """
1: lo: <LOOPBACK,UP,LOWER_UP> mtu 65536 qdisc noqueue state UNKNOWN mode DEFAULT group default qlen 1000\\    link/loopback 00:00:00:00:00:00 brd 00:00:00:00:00:00
2: enp1s0: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc fq_codel state UP mode DEFAULT group default qlen 1000\\    link/ether 52:54:00:13:14:b5 brd ff:ff:ff:ff:ff:ff
3: enp8s0: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc fq_codel state UP mode DEFAULT group default qlen 1000\\    link/ether 52:54:00:e5:11:d4 brd ff:ff:ff:ff:ff:ff
4: enp1s0.2@enp1s0: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc noqueue state UP mode DEFAULT group default qlen 1000\\    link/ether 52:54:00:13:14:b5 brd ff:ff:ff:ff:ff:ff
5: ib0: <NO-CARRIER,BROADCAST,MULTICAST,UP> mtu 4092 qdisc mq state DOWN group default qlen 256\\    link/infiniband 00:01:02:03:fd:90:0:00:00:00:00:00:ef:0d:8b:02:01:d9:82:fd
"""

IP_LINK_BAD = ""

NMCLI_C_SHOW_OUTPUT_1 = """
NAME      UUID                                  TYPE      DEVICE
enp0s3    320d4923-c410-4b22-b7e9-afc5f794eecc  ethernet  enp0s3
virbr0    7c7dec66-4a8c-4b49-834a-889194b3b83c  bridge    virbr0
test-net  f858b1cc-d149-4de0-93bc-b1826256847a  ethernet  --
team0     bf000427-d9f1-432f-819d-257edb86c6fb  team      team0
ens3      1b1f5c95-1026-4699-95e5-2cd5baa033c3  ethernet  ens3
ens8      a2c39643-b356-435a-955c-50cef6b36052  ethernet  ens8
team1     ca07b1cf-b293-4871-b255-17f1abfa991d  team      team1
"""

NMCLI_C_SHOW_OUTPUT_2 = """
NAME      UUID                                  TYPE      DEVICE
enp0s3    320d4923-c410-4b22-b7e9-afc5f794eecc  ethernet  enp0s3
"""

NMCLI_C_SHOW_OUTPUT_3 = """
NAME      UUID                                  TYPE      DEVICE
team0     bf000427-d9f1-432f-819d-257edb86c6fb  team      --
"""

NMCLI_C_SHOW_EMPTY = """
NAME      UUID                                  TYPE      DEVICE
"""

EXPECTED = ['enp1s0', 'enp8s0', 'enp1s0.2']


def test_ethernet_interfaces():
    ip_link_command = Mock()
    ip_link_command.content = IP_LINK.splitlines()
    broker = {LocalSpecs.ip_link: ip_link_command}
    result = interfaces(broker)
    assert result is not None
    assert isinstance(result, list)
    assert result == sorted(EXPECTED)


def test_ethernet_interfaces_bad():
    ip_link_command = Mock()
    ip_link_command.content = IP_LINK_BAD.splitlines()
    broker = {LocalSpecs.ip_link: ip_link_command}
    with pytest.raises(SkipComponent):
        interfaces(broker)


def test_team_device_1():
    mmcli_c_show = NmcliConnShow(context_wrap(NMCLI_C_SHOW_OUTPUT_1))

    broker = {
        NmcliConnShow: mmcli_c_show
    }
    result = team_interfaces(broker)
    assert result is not None
    assert isinstance(result, list)
    assert result == sorted(['team0', 'team1'])


def test_team_device_2():
    mmcli_c_show = NmcliConnShow(context_wrap(NMCLI_C_SHOW_OUTPUT_2))

    broker = {
        NmcliConnShow: mmcli_c_show
    }
    with pytest.raises(SkipComponent):
        team_interfaces(broker)


def test_team_device_3():
    mmcli_c_show = NmcliConnShow(context_wrap(NMCLI_C_SHOW_OUTPUT_3))

    broker = {
        NmcliConnShow: mmcli_c_show
    }
    with pytest.raises(SkipComponent):
        team_interfaces(broker)


def test_team_device_bad():
    mmcli_c_show = NmcliConnShow(context_wrap(NMCLI_C_SHOW_EMPTY))

    broker = {
        NmcliConnShow: mmcli_c_show
    }
    with pytest.raises(SkipComponent):
        team_interfaces(broker)
